#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module contains the graphics factory for the jdraw file format"""

from __future__ import absolute_import

from builtins import str
from builtins import range

import os
import traceback

from taurus.external.qt import Qt
from taurus.core.util.log import Logger
from taurus.core.util.singleton import Singleton
from taurus.core.util.containers import CaselessDict
from taurus.qt.qtgui.graphic import (TaurusBaseGraphicsFactory,
                                     TaurusGraphicsScene, TaurusGraphicsItem)


__all__ = ["TaurusJDrawGraphicsFactory"]

__docformat__ = 'restructuredtext'


LINESTYLE_JDW2QT = {0: Qt.Qt.SolidLine,
                    1: Qt.Qt.DotLine,
                    2: Qt.Qt.DashLine,
                    3: Qt.Qt.DashLine,
                    4: Qt.Qt.DashDotLine}

FILLSTYLE_JDW2QT = {0: Qt.Qt.NoBrush,
                    1: Qt.Qt.SolidPattern,
                    2: Qt.Qt.FDiagPattern,
                    3: Qt.Qt.BDiagPattern,
                    4: Qt.Qt.DiagCrossPattern,
                    5: Qt.Qt.FDiagPattern,
                    6: Qt.Qt.BDiagPattern,
                    7: Qt.Qt.Dense5Pattern,
                    8: Qt.Qt.Dense7Pattern,
                    9: Qt.Qt.Dense6Pattern,
                    10: Qt.Qt.Dense4Pattern,
                    11: Qt.Qt.LinearGradientPattern}

TEXTHINT_JDW2QT = CaselessDict({
    'helvetica': Qt.QFont.Helvetica,
    'serif': Qt.QFont.Serif,
    'sansserif': Qt.QFont.SansSerif,
    'courier': Qt.QFont.Courier,
    'Monospaced': Qt.QFont.Courier,
    'times': Qt.QFont.Times,
    '': Qt.QFont.AnyStyle, })

ALIGNMENT = {
    0: Qt.Qt.AlignHCenter,
    1: Qt.Qt.AlignLeft,
    2: Qt.Qt.AlignRight,
}

VALIGNMENT = {
    0: Qt.Qt.AlignVCenter,
    1: Qt.Qt.AlignTop,
    2: Qt.Qt.AlignBottom,
}


class TaurusJDrawGraphicsFactory(Singleton, TaurusBaseGraphicsFactory, Logger):

    def __init__(self, parent, alias=None, delayed=False):
        """ Initialization. Nothing to be done here for now."""
        self.myparent = parent
        self.call__init__wo_kw(TaurusBaseGraphicsFactory, parent)
        self._zBufferLevel = 0
        self._delayed = delayed
        self.alias = alias if alias is not None else {}

    def init(self, *args, **kwargs):
        """Singleton instance initialization."""
        self.call__init__(Logger, self.__class__.__name__)
        self.call__init__(TaurusBaseGraphicsFactory)

    def getZBufferLevel(self):
        return self._zBufferLevel

    def incZBufferLevel(self):
        self._zBufferLevel += 1
        return self._zBufferLevel

    def setZBufferLevel(self, level):
        self._zBufferLevel = level

    def resetZBufferLevel(self):
        self.setZBufferLevel(0)

    def getSceneObj(self, items):
        scene = TaurusGraphicsScene(self.myparent)
        for item in items:
            try:
                if isinstance(item, Qt.QWidget):
                    scene.addWidget(item)
                elif isinstance(item, Qt.QGraphicsItem):
                    scene.addItem(item)
            except:
                self.warning("Unable to add item %s to scene" % str(item))
                self.debug("Details:", exc_info=1)
        return scene

    def getObj(self, name, params):
        method_name = 'get' + name.lstrip('JD') + 'Obj'
        try:
            method = getattr(self, method_name)
            obj = method(params)
            obj.setZValue(self.incZBufferLevel())
            return obj
        except:
            self.warning("Error fetching object")
            self.info("Details:", exc_info=1)
            pass
        return None

    def getRectangleObj(self, params):
        item = self.getGraphicsItem('Rectangle', params)
        x1, y1, x2, y2 = params.get('summit')
        width = x2 - x1
        height = y2 - y1
        # item.setPos(x1,y1)
        item.setRect(x1, y1, width, height)

        return item

    def getRoundRectangleObj(self, params):
        item = self.getGraphicsItem('RoundRectangle', params)
        x1, y1, x2, y2 = params.get('summit')
        width = x2 - x1
        height = y2 - y1
        cornerWidth = params.get('cornerWidth', 24)
        nbPoints = params.get('step', 6)
        # item.setPos(x1,y1)
        item.setRect(x1, y1, width, height)
        item.setCornerWidth(cornerWidth, nbPoints)

        return item

    def getLineObj(self, params):
        item = self.getGraphicsItem('Line', params)
        x1, y1, x2, y2 = params.get('summit')
        item.setLine(x1, y1, x2, y2)

        return item

    def getEllipseObj(self, params):
        item = self.getGraphicsItem('Ellipse', params)

        x1, y1, x2, y2 = params.get('summit')
        width = x2 - x1
        height = y2 - y1
        item.setRect(x1, y1, width, height)

        return item

    def getPolylineObj(self, params):
        item = self.getGraphicsItem('Polyline', params)

        polygon = Qt.QPolygonF()
        p = params.get('summit')
        for i in range(0, len(p), 2):
            polygon.append(Qt.QPointF(p[i], p[i + 1]))
        item.setPolygon(polygon)

        return item

    def getSplineObj(self, params):
        item = self.getGraphicsItem('Spline', params)
        p = params.get('summit')
        p = [Qt.QPointF(p[i], p[i + 1]) for i in range(0, len(p), 2)]
        item.setControlPoints(p)
        isClosed = params.get('isClosed', True)
        item.setClose(isClosed)
        return item

    def getLabelObj(self, params):
        item = self.getGraphicsItem('Label', params)
        self.readLabelObj(item, params)
        return item

    def readLabelObj(self, item, params):
        origin = params.get('origin')
        item.setPos(origin[0], origin[1])

        summit = params.get('summit')
        x, y = summit[0] - origin[0], summit[1] - origin[1]
        width, height = summit[2] - summit[0], summit[3] - summit[1]
        item.setRect(x, y, width, height)

        # it is parsed as a float
        vAlignment = int(params.get('vAlignment', 0))
        hAlignment = int(params.get('hAlignment', 0))
        assert(vAlignment in VALIGNMENT)
        assert(hAlignment in ALIGNMENT)
        vAlignment = VALIGNMENT[vAlignment]
        hAlignment = ALIGNMENT[hAlignment]
        item.setAlignment(hAlignment | vAlignment)

        fnt = params.get('font', None)
        if fnt:
            family, style, size = fnt
            f = Qt.QFont(family, int(.85 * size), Qt.QFont.Light, False)
            f.setStyleHint(TEXTHINT_JDW2QT.get(family, Qt.QFont.AnyStyle))
            f.setStyleStrategy(Qt.QFont.PreferMatch)
            if style == 1:
                f.setWeight(Qt.QFont.DemiBold)
            elif style == 2:
                f.setItalic(True)
            elif style == 3:
                f.setWeight(Qt.QFont.DemiBold)
                f.setItalic(True)
            # TODO: Improve code in order to be able to set a suitable font
            item.setFont(f)
        fg = params.get("foreground", (0, 0, 0))
        color = Qt.QColor(fg[0], fg[1], fg[2])
        item.setDefaultTextColor(color)

        txt = params.get('text')
        if txt:
            if any(isinstance(txt, t) for t in (list, tuple, set)):  # Parsing several lines of text
                txt = '\n'.join(txt)
            item.setPlainText(str(txt))
            item._currText = txt

    def getGroupObj(self, params):
        item = self.getGraphicsItem('Group', params)
        s = params.get('summit')
        x1, y1 = s[0], s[1]
        item.setPos(x1, y1)
        children = params.get('children')
        if children:
            for child in children:
                if child:
                    #self.info('jdraw.py: "%s".addItem("%s")'%(str(params.get('name')),str(child)))
                    item.addToGroup(child)
        if item._fillStyle:
            self.set_item_filling(item, expand=True)
        return item

    def getSwingObjectObj(self, params):
        item = self.getGraphicsItem('SwingObject', params)
        s = params.get('summit')
        x1, y1 = s[0], s[1]
        item.setPos(x1, y1)

        className = params.get('className')
        if className == "fr.esrf.tangoatk.widget.attribute.SimpleScalarViewer":
            self.readSimpleScalarViewerObj(item, params)

        return item

    def readSimpleScalarViewerObj(self, item, params):
        self.readLabelObj(item, params)

        ext = params.get('extensions')

        c = ext.get('validBackground')
        c = [int(x) for x in c.split(",")]
        if c:
            validBackground = Qt.QColor(*c)
            item.setValidBackground(validBackground)

        invalidText = ext.get('invalidText', "-----")
        item.setNoneValue(invalidText)

        alarmEnabled = ext.get('alarmEnabled', True)
        alarmEnabled = alarmEnabled.lower().strip() in ["yes", "true", "1"]
        item.setShowQuality(alarmEnabled)

        unitVisible = ext.get('unitVisible', True)
        unitVisible = unitVisible.lower().strip() in ["yes", "true", "1"]
        item.setUnitVisible(unitVisible)

        userFormat = ext.get('userFormat', None)
        item.setUserFormat(userFormat)

    def getImageObj(self, params):
        item = self.getGraphicsItem('Image', params)
        s = params.get('summit')
        x1, y1, x2, y2 = s
        item.setPos(x1, y1)

        fname = params.get('file_name')
        if fname:
            if os.path.isfile(fname):
                fname = os.path.realpath(fname)
            elif hasattr(self.myparent, 'path'):
                #self.info('using path param ...')
                fname = self.myparent.path + os.path.sep + fname
            self.debug('Opening JDImage(%s) = x,y,w,h=%f,%f,%f,%f' %
                       (fname, x1, y1, x2 - x1, y2 - y1))
            pixmap = Qt.QPixmap(fname)
            item.setPixmap(pixmap.scaled(x2 - x1, y2 - y1))
            #item.scale(float(w)/pixmap.width(), float(h)/pixmap.height())
        else:
            self.warning('No filename for image!?!')
        return item

    def set_common_params(self, item, params):
        if not item:
            return
        item._params = params
        name = params.get('name')

        if name.lower() == "ignorerepaint":
            name = ""
            if not 'extensions' in params:
                params['extensions'] = {}
            params.get('extensions')["ignoreRepaint"] = "true"

        if self.alias:
            for k, v in self.alias.items():
                name = str(name).replace(k, v)

        # Forcing not-Taurus items to have a name and be able to trigger events
        setattr(item, '_name', name)
        if name and not self._delayed:
            if isinstance(item, TaurusGraphicsItem):
                #self.debug('TaurusJDrawGraphicsFactory.set_common_params(): %s.setModel(%s)'%(item,name))
                item.setModel(name)
            else:
                self.debug('TaurusJDrawGraphicsFactory.set_common_params(%s): %s is not a TaurusGraphicsItem' % (
                    name, type(item).__name__))

        visibilitymapper = params.get('visibilitymapper')
        if not visibilitymapper is None:
            mapping_type = visibilitymapper['mapping_type']
            mode = visibilitymapper['mode']
            default = visibilitymapper['default']
            item._default = default
            item._map = visibilitymapper['map']

        visible = params.get('visible')
        if not visible is None:
            item.setVisible(visible)

        extensions = params.get('extensions')
        if extensions:
            item._extensions = extensions

        try:
            getattr(item, 'setPen')
            fg = params.get("foreground", (0, 0, 0))
            lineWidth = params.get("lineWidth", 1)
            if lineWidth == 0:
                pen = Qt.QPen(Qt.Qt.NoPen)
            else:
                pen = Qt.QPen(Qt.QColor(fg[0], fg[1], fg[2]))
                pen.setWidth(lineWidth)
                pen.setStyle(LINESTYLE_JDW2QT[params.get("lineStyle", 0)])
            item.setPen(pen)
        except AttributeError as ae:
            pass
        except Exception as e:
            self.warning('jdraw.set_common_params(%s(%s)).(foreground,width,style) failed!: \n\t%s' % (
                type(item).__name__, name, traceback.format_exc()))

        fillStyle = FILLSTYLE_JDW2QT[params.get('fillStyle', 0)]
        item._fillStyle = fillStyle

        if hasattr(item, 'brush'):
            brush = Qt.QBrush()
            if fillStyle == Qt.Qt.LinearGradientPattern:
                ox, oy = params.get('origin', (0, 0))
                gradient = Qt.QLinearGradient(ox + params.get('gradX1', 0),
                                              oy + params.get('gradY1', 0),
                                              ox + params.get('gradX2', 0),
                                              oy + params.get('gradY2', 0))
                c = params.get('gradC1', (0, 0, 0))
                gradient.setColorAt(0, Qt.QColor(c[0], c[1], c[2]))
                c = params.get('gradC2', (255, 255, 255))
                gradient.setColorAt(1, Qt.QColor(c[0], c[1], c[2]))

                gradCyclic = params.get('gradCyclic', False)
                if gradCyclic:
                    gradient.setSpread(Qt.QGradient.ReflectSpread)

                brush = Qt.QBrush(gradient)
            else:
                brush.setStyle(fillStyle)

            bg = params.get('background', (255, 255, 255))
            brush.setColor(Qt.QColor(bg[0], bg[1], bg[2]))
            item.setBrush(brush)

    def set_item_filling(self, item, pattern=Qt.Qt.Dense4Pattern, expand=False):
        count = 0
        item._fillStyle = item._fillStyle or pattern
        if hasattr(item, 'brush'):
            br = item.brush()
            br.setStyle(item._fillStyle)
            item.setBrush(br)
        if expand:
            for c in item.childItems():
                if not c._fillStyle:
                    self.set_item_filling(c, pattern=pattern, expand=True)
        return

if __name__ == "__main__":
    from . import jdraw_view
    jdraw_view.jdraw_view_main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Do not use. Deprecated. Backwards compatibility module for transition from
TaurusDatabase to TaurusAuthority"""


from logging import warn
warn('taurusdatabase module is deprecated. Use taurusauthority instead')

import traceback
traceback.print_stack()

from .taurusauthority import *

TaurusDatabase = TaurusAuthority

# The following block is commented out because it produces a circular import
# try:
#     from taurus.core.tango.tangodatabase import TangoInfo as TaurusInfo
#     from taurus.core.tango.tangodatabase import TangoAttrInfo as TaurusAttrInfo
#     from taurus.core.tango.tangodatabase import TangoDevInfo as TaurusDevInfo
#     from taurus.core.tango.tangodatabase import TangoDevClassInfo as \
#                                                             TaurusDevClassInfo
#     from taurus.core.tango.tangodatabase import TangoServInfo as TaurusServInfo
# except ImportError, e:
#     warn('taurusdatabase: Cannot import tango info objects: %s', repr(e))
#



# -*- coding: utf-8 -*-

##############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""
This module provides a simple dialog to display typical About
<application> dialog. It will create a Dialog with the title being
*Dialog + <app name>* and a default text combining the application
name and version, organization name and domain.

This behaviour can be changed by setting the dialog window title
(:meth:`~AboutDialog.setWindowTitle`) and content
(:meth:`~AboutDialog.setText`, :meth:`~AboutDialog.setHtml`)

Example usage::

    from taurus.external.qt import Qt
    from taurus.qt.qtgui.help import AboutDialog

    app = Qt.QApplication([])
    app.setApplicationName("Example GUI")
    app.setApplicationVersion("1.2.3")
    app.setOrganizationName("Taurus")
    app.setOrganizationDomain("http://www.taurus-scada.org/")
    about_dialog = AboutDialog()
    pixmap = Qt.QIcon.fromTheme("folder-open").pixmap(64, 64)
    about_dialog.setPixmap(pixmap)
    about_dialog.exec_()

    """

__all__ = ["AboutDialog"]

from taurus.external.qt import Qt
from taurus.qt.qtgui.util.ui import UILoadable


@UILoadable
class AboutDialog(Qt.QDialog):
    """
    Simple dialog to display typical About <application> dialog.
    It will create a Dialog with the title being
    *Dialog + <app name>* and a default text combining the application
    name and version, organization name and domain.

    This behaviour can be changed by setting the dialog window title
    (:meth:`~AboutDialog.setWindowTitle`) and content
    (:meth:`~AboutDialog.setText`, :meth:`~AboutDialog.setHtml`)

    Example usage::

        from taurus.external.qt import Qt
        from taurus.qt.qtgui.help import AboutDialog

        app = Qt.QApplication([])
        app.setApplicationName("Example GUI")
        app.setApplicationVersion("1.2.3")
        app.setOrganizationName("Taurus")
        app.setOrganizationDomain("http://www.taurus-scada.org/")
        about_dialog = AboutDialog()
        pixmap = Qt.QIcon.fromTheme("folder-open").pixmap(64, 64)
        about_dialog.setPixmap(pixmap)
        about_dialog.exec_()

    """

    _Template = "<html><body><p><b>{0}</b></p>" \
                "<p>{1}</p><p>{2}</p>" \
                "<p><a href=\"{3}\">{2}</a></p>" \
                "(c) Copyright {2}"

    def __init__(self, parent=None):
        Qt.QDialog.__init__(self, parent)
        self.loadUi()
        self.text_browser.setFrameStyle(Qt.QFrame.NoFrame)
        palette = Qt.QPalette()
        palette.setColor(Qt.QPalette.Base,
                         palette.color(Qt.QPalette.Background))
        self.text_browser.setPalette(palette)
        self.logo_widget.setAlignment(Qt.Qt.AlignHCenter |
                                      Qt.Qt.AlignTop)
        name = Qt.qApp.applicationName()
        version = Qt.qApp.applicationVersion()
        org = Qt.qApp.organizationName()
        org_domain = Qt.qApp.organizationDomain()
        self.setWindowTitle("About " + Qt.qApp.applicationName())
        self.setHtml(self._Template.format(name, version, org,
                                           org_domain))

    def setText(self, text):
        """
        Sets the dialog text.

        :param text: new text
        :type text: str
        """
        self.text_browser.setText(text)

    def getHtml(self):
        """
        Gets the current dialog HTML text.

        :return: the current dialog HTML text.
        :rtype: str
        """
        return self.text_browser.toHtml()

    @Qt.Slot(str)
    def setHtml(self, html):
        """
        Sets the dialog HTML text.

        :param text: new HTML text
        :type text: str
        """
        self.text_browser.setHtml(html)

    def resetHtml(self):
        """
        Resets the dialog HTML to an empty HTML document
        """
        self.setHtml("<html><body></body></html>")

    def getSource(self):
        """
        Gets the current dialog document source.

        :return: the current dialog document source.
        :rtype: Qt.QUrl
        """
        return self.text_browser.source()

    @Qt.Slot(Qt.QUrl)
    def setSource(self, source):
        """
        Sets the dialog document source.

        :param text: new document source
        :type text: Qt.QUrl
        """
        self.text_browser.setSource(source)

    @Qt.Slot(Qt.QPixmap)
    def setPixmap(self, pixmap):
        """
        Sets the dialog pixmap

        :param text: new pixmap
        :type text: Qt.QPixmap
        """
        self.logo_widget.setPixmap(pixmap)

    def getPixmap(self):
        """
        Gets the current pixmap.

        :return: the current dialog pixmap
        :rtype: Qt.QPixmap
        """
        pixmap = self.logo_widget.getPixmap()
        if pixmap is None:
            pixmap = Qt.QPixmap()
        return pixmap

    def resetPixmap(self):
        """
        Resets the dialog pixmap to a Null pixmap.
        """
        self.setPixmap(Qt.QPixmap())

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return {'group': 'Taurus Help',
                'icon': Qt.QIcon.fromTheme("help"),
                'module': 'taurus.qt.qtgui.help',
                'container': False}

    #: This property holds the current dialog pixmap
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`~AboutDialog.getPixmap`
    #:     * :meth:`~AboutDialog.setPixmap`
    #:     * :meth:`~AboutDialog.resetPixmap`
    #:
    pixmap = Qt.Property("QPixmap", getPixmap, setPixmap, resetPixmap)

    #: This property holds the current dialog HTML
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`~AboutDialog.getHtml`
    #:     * :meth:`~AboutDialog.setHtml`
    #:     * :meth:`~AboutDialog.resetHtml`
    #:
    html = Qt.Property("QString", getHtml, setHtml, resetHtml)

    #: This property holds the current dialog document source
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`~AboutDialog.getSource`
    #:     * :meth:`~AboutDialog.setSource`
    #:
    source = Qt.Property("QUrl", getSource, setSource)


def main():
    app = Qt.QApplication([])
    app.setApplicationName("Example GUI")
    app.setApplicationVersion("1.2.3")
    app.setOrganizationName("Taurus")
    app.setOrganizationDomain("http://www.taurus-scada.org/")
    about_dialog = AboutDialog()
#    about_dialog.setText(txt)
    pixmap = Qt.QIcon.fromTheme("folder-open").pixmap(64, 64)
    about_dialog.setPixmap(pixmap)
    about_dialog.exec_()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides Qt color management for taurus"""

__all__ = ["QtColorPalette",
           "QT_DEVICE_STATE_PALETTE",
           "QT_ATTRIBUTE_QUALITY_PALETTE",
           ]

__docformat__ = 'restructuredtext'

from taurus.external.qt import Qt

from taurus.core.util.colors import ColorPalette, \
    DEVICE_STATE_DATA, ATTRIBUTE_QUALITY_DATA
from taurus.core.taurusbasetypes import AttrQuality
from taurus.core.util.log import deprecation_decorator


class QtColorPalette(ColorPalette):

    def __init__(self, dat, int_decoder_dict):
        ColorPalette.__init__(self, dat, int_decoder_dict)
        self._qcolor_cache_fg = dict()
        self._qcolor_cache_bg = dict()
        self._qbrush_cache_fg = dict()
        self._qbrush_cache_bg = dict()

    def qbrush(self, stoq):
        # print stoq
        """Returns the brush for the specified state or quality"""
        name = self._decoder(stoq)

        f = self._qbrush_cache_fg
        b = self._qbrush_cache_bg
        if name not in f:
            f[name] = Qt.QBrush(self.qcolor(stoq)[1])

        if name not in b:
            b[name] = Qt.QBrush(self.qcolor(stoq)[0])
            if name == 'None':
                b[name].setStyle(Qt.Qt.BDiagPattern)

        return (b[name], f[name])

    def qcolor(self, stoq):
        """Returns the color for the specified state or quality"""
        name = self._decoder(stoq)

        f = self._qcolor_cache_fg
        b = self._qcolor_cache_bg
        if name not in f:
            f[name] = Qt.QColor(self.number(name, True))

        if name not in b:
            b[name] = Qt.QColor(self.number(name))

        return (b[name], f[name])

    @deprecation_decorator(alt='QtColorPalette.qcolor()', rel='4.5')
    def qvariant(self, stoq):
        """Returns the color for the specified state or quality"""
        return self.qcolor(stoq)



QT_ATTRIBUTE_QUALITY_PALETTE = QtColorPalette(ATTRIBUTE_QUALITY_DATA,
                                              AttrQuality)

try:
    # TODO: Tango-centric
    from taurus.core.tango import DevState
    QT_DEVICE_STATE_PALETTE = QtColorPalette(DEVICE_STATE_DATA, DevState)
except ImportError:
    QT_DEVICE_STATE_PALETTE = QtColorPalette(DEVICE_STATE_DATA, {})



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Test for taurus.core.util.codecs"""

#__all__ = []

__docformat__ = 'restructuredtext'

import copy
import unittest
from taurus.test import insertTest
from taurus.core.util.codecs import CodecFactory
import numpy


@insertTest(helper_name='encDec', cname='json', data=[1, 2, 3])
@insertTest(helper_name='encDec', cname='zip', data=b'foobar')
@insertTest(helper_name='encDec', cname='zip_json', data=[1, 2, 3])
@insertTest(helper_name='encDec', cname='videoimage',
            data=numpy.ones((2, 2), dtype='uint8'))
@insertTest(helper_name='encDec', cname='zip_null_zip_videoimage',
            data=numpy.ones((2, 2), dtype='uint8'))
@insertTest(helper_name='dec', cname='videoimage',
            data=b'VDEO\x00\x01\x00\x07\x00\x00\x00\x00\x00\x00\x00' +
            b'\x01\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00 ' +
            b'\x00\x00\x00\x00\x01\x01\x01\x01\x01\x01\x01\x01' +
            b'\x01\x01\x01\x01\x01\x01\x01\x01',
            expected=numpy.ones((2, 2, 3), dtype='uint8'))
class CodecTest(unittest.TestCase):
    '''TestCase for checking codecs'''

    def encDec(self, cname=None, data=None, expected=None):
        '''Check that data can be encoded-decoded properly'''
        if expected is None:
            expected = copy.deepcopy(data)
        _, enc = self.enc(cname=cname, data=data)
        self.dec(cname=cname, data=enc, expected=expected)

    def enc(self, cname=None, data=None, expected=None):
        '''Check that data can be encoded-decoded properly'''
        cf = CodecFactory()
        codec = cf.getCodec(cname)
        fmt, enc = codec.encode(('', data))
        if expected is not None:
            msg = ('Wrong data after encoding with %s:\n' +
                   ' -expected:%s\n -obtained:%s') % (cname, expected, enc)
            if numpy.isscalar(expected):
                equal = enc == expected
            else:
                equal = numpy.all(enc == expected)
            self.assertTrue(equal, msg)
        return fmt, enc

    def dec(self, cname=None, data=None, expected=None):
        '''Check that data can be encoded-decoded properly'''
        cf = CodecFactory()
        codec = cf.getCodec(cname)
        fmt, dec = codec.decode((cname, data))
        if expected is not None:
            msg = ('Wrong data after decoding with %s:\n' +
                   ' -expected:%s\n -obtained:%s') % (cname, expected, dec)
            if numpy.isscalar(expected):
                equal = dec == expected
            else:
                equal = numpy.all(dec == expected)
            self.assertTrue(equal, msg)
        return fmt, dec

if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-

##############################################################################
##
## This file is part of Taurus
##
## http://taurus-scada.org
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Taurus is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Taurus is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################


from taurus.core.util import log as __log

__log.deprecated(dep='taurus.external.argparse', rel='4.3.2',
                 alt='argparse')

from argparse import *



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
This is a variation on "Constants in Python" by Alex Martelli, from which the
solution idea was borrowed, and enhanced according suggestions of Zoran Isailovski.

In Python, any variable can be re-bound at will -- and modules don't let you
define special methods such as an instance's __setattr__ to stop attribute
re-binding. Easy solution (in Python 2.1 and up): use an instance as "module"...

In Python 2.1 and up, no check is made any more to force entries in sys.modules
to be actually module objects. You can install an instance object there and take
advantage of its attribute-access special methods (e.g., as in this snippet, to
prevent type rebindings.

Usage::

  import consttype
  consttype.magic = 23    # Bind an attribute to a type ONCE
  consttype.magic = 88    # Re-bind it to a same type again
  consttype.magic = "one" # But NOT re-bind it to another type: this raises consttype._ConstError
  del consttype.magic     # Remove an named attribute
  consttype.__del__()     # Remove all attributes
"""

from builtins import object

__docformat__ = "restructuredtext"


class _consttype(object):

    class _ConstTypeError(TypeError):
        pass

    def __repr__(self):
        return "Constant type definitions."

    def __setattr__(self, name, value):
        v = self.__dict__.get(name, value)
        if type(v) is not type(value):
            raise self._ConstTypeError("Can't rebind %s to %s" % (
                type(v), type(value)))
        self.__dict__[name] = value

    def __del__(self):
        self.__dict__.clear()

import sys
sys.modules[__name__] = _consttype()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
qlogo.py:
"""

__all__ = ["QLogo"]

__docformat__ = 'restructuredtext'

from taurus import tauruscustomsettings
from taurus.external.qt import Qt
from taurus.qt.qtgui.icon import getCachedPixmap


class QLogo(Qt.QLabel):

    def __init__(self, parent=None, designMode=False):
        self.__name = self.__class__.__name__
        Qt.QLabel.__init__(self, parent)
        sizePolicy = Qt.QSizePolicy(
            Qt.QSizePolicy.Policy(0), Qt.QSizePolicy.Policy(0))
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizePolicy)
        self.setScaledContents(True)
        logo = getattr(tauruscustomsettings, 'ORGANIZATION_LOGO',
                       "logos:taurus.png")
        self.setPixmap(getCachedPixmap(logo))

    @classmethod
    def getQtDesignerPluginInfo(cls):
        """Returns pertinent information in order to be able to build a valid
        QtDesigner widget plugin.

        The dictionary returned by this method should contain *at least* the
        following keys and values:

            - 'module' : a string representing the full python module name (ex.: 'taurus.qt.qtgui.base')
            - 'icon' : a string representing valid resource icon (ex.: 'designer:combobox.png')
            - 'container' : a bool telling if this widget is a container widget or not.

        This default implementation returns the following dictionary:

            { 'module'    : 'taurus.qt.qtgui.base',
              'group'     : 'Taurus Widgets',
              'icon'      : 'logos:taurus.svg',
              'container' : False }

        :return: (dict) a map with pertinent designer information"""
        return {
            'group': 'Taurus Widgets',
            'icon': 'logos:taurus.png',
            'container': False}


def main():
    import sys
    from taurus.qt.qtgui.application import TaurusApplication

    app = TaurusApplication(sys.argv)
    w = QLogo()
    w.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus, a Tango User Interface Library
##
# http://www.tango-controls.org/static/taurus/latest/doc/html/index.html
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

from __future__ import print_function
import sys

import taurus.core.util
import taurus.qt.qtgui.util
import taurus.qt.qtgui.application

from taurus.external.qt import Qt


class TaurusDemoPanel(Qt.QWidget):

    def __init__(self, parent=None):
        Qt.QWidget.__init__(self, parent)
        self._groups = {}

        layout = Qt.QGridLayout()
        self.setLayout(layout)

        wf = taurus.qt.qtgui.util.TaurusWidgetFactory()

        taurus_widgets = wf.getWidgets()

        demos = {}
        for widget_name in taurus_widgets:
            widget_module_name, widget_class = taurus_widgets[widget_name]
            internal_widget_module_name = widget_class.__module__
            if internal_widget_module_name in demos:
                continue
            internal_widget_module = sys.modules[internal_widget_module_name]
            if hasattr(internal_widget_module, "demo"):
                if hasattr(internal_widget_module.demo, '__call__'):
                    demos[internal_widget_module_name] = internal_widget_module.demo

        groups = set()

        for demo_name in demos:
            parts = demo_name.split(".")
            group = parts[-2]
            groups.add(group)

        for group in sorted(groups):
            self.addGroup(group)

        for demo_name in sorted(demos):
            demo_func = demos[demo_name]
            parts = demo_name.split(".")
            group = parts[-2]
            if not demo_func.__doc__:
                continue
            try:
                self.addDemo(demo_func.__doc__, demo_func, group)
            except Exception as e:
                print(80 * "-")
                print("Problems adding demo", demo_name)
                print(e)

    def addGroup(self, name):
        g = Qt.QGroupBox(name)
        layout = self.layout()
        layout.addWidget(g, layout.rowCount(), 0)
        l = Qt.QGridLayout()
        g.setLayout(l)
        self._groups[name] = g

    def addDemo(self, name, f, group):
        g = self._groups[group]
        layout = g.layout()
        row = layout.rowCount()
        #label = Qt.QLabel(name, self)
        button = Qt.QPushButton(name, self)
        button._f = f
        layout.addWidget(button, row, 0)
        button.clicked.connect(self.go)

    def go(self):
        b = self.sender()
        f = b._f
        dialog = None
        try:
            w = f()
            if w is None or not isinstance(w, Qt.QWidget):
                raise Exception("demo function does not return a valid widget")
            dialog = Qt.QDialog()
            layout = Qt.QVBoxLayout()
            dialog.setLayout(layout)
            layout.addWidget(w)
            dialog.exec_()
        except Exception as e:
            if dialog is not None:
                dialog.done(0)
                dialog.hide()
                dialog = None
            print(str(e))
            return
            d = Qt.QErrorMessage()
            d.showMessage(str(e))


def main():
    import taurus.core.util.argparse
    parser = taurus.core.util.argparse.get_taurus_parser()
    parser.set_description("A demo application for taurus")
    app = taurus.qt.qtgui.application.TaurusApplication(cmd_line_parser=parser,
                                                        app_name="taurusdemo",
                                                        app_version="1.0",
                                                        org_domain="Taurus",
                                                        org_name="Tango community")
    gui = TaurusDemoPanel()
    gui.setWindowTitle(app.applicationName())
    gui.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
safeeval.py: Safe eval replacement with whitelist support
"""
from __future__ import print_function

from builtins import range
from builtins import object

__all__ = ["SafeEvaluator"]

__docformat__ = "restructuredtext"


class SafeEvaluator(object):
    """This class provides a safe eval replacement.

    The method eval() will only evaluate the expressions considered safe (whitelisted).
    By default it has a whitelist of mathematical expressions that can be turn off using defaultSafe=False at init

    The user can add more safe functions passing a safedict to the addSafe() or init methods.

    Functions can be removed by name using removeSafe()

    Note: In order to use variables defined outside, the user must explicitly declare them safe.
    """

    def __init__(self, safedict=None, defaultSafe=True):
        self._default_numpy = ('abs', 'array', 'arange', 'arccos', 'arcsin', 'arctan', 'arctan2', 'average',
                               'ceil', 'cos', 'cosh', 'degrees', 'dot', 'e', 'exp', 'fabs', 'floor', 'fmod',
                               'frexp', 'hypot', 'ldexp', 'linspace', 'log', 'log10', 'logspace',
                               'modf', 'ones', 'pi', 'radians', 'shape', 'sin', 'sinh', 'sqrt', 'sum', 'tan',
                               'tanh', 'zeros')
        self._default_numpy_random = ('randn', 'rand', 'randint')

        if safedict is None:
            safedict = {}
        self.safe_dict = safedict
        if defaultSafe:
            import numpy
            from taurus.core.units import Quantity, UR
            self.safe_dict['pow'] = pow
            self.safe_dict['len'] = len
            self.safe_dict['int'] = int
            self.safe_dict['float'] = float
            self.safe_dict['bool'] = bool
            self.safe_dict['str'] = str
            self.safe_dict['bytes'] = bytes
            self.safe_dict['list'] = list
            self.safe_dict['range'] = range
            self.safe_dict['True'] = True
            self.safe_dict['False'] = False
            self.safe_dict['None'] = None
            for n in self._default_numpy:
                self.safe_dict[n] = getattr(numpy, n)
            for n in self._default_numpy_random:
                self.safe_dict[n] = getattr(numpy.random, n)
            self.safe_dict['Quantity'] = Quantity
            self.safe_dict['Q'] = Quantity  # Q() is an alias for Quantity()
            self.safe_dict['UR'] = UR

        self._originalSafeDict = self.safe_dict.copy()

    def eval(self, expr):
        """safe eval"""
        return eval(expr, {"__builtins__": None}, self.safe_dict)

    def addSafe(self, safedict, permanent=False):
        """The values in safedict will be evaluable (whitelisted)
        The safedict is as follows: {"eval_name":object, ...}. The evaluator will interpret eval_name as object.
        """
        self.safe_dict.update(safedict)
        if permanent:
            self._originalSafeDict.update(safedict)

    def removeSafe(self, name, permanent=False):
        """Removes an object from the whitelist"""
        self.safe_dict.pop(name)
        if permanent:
            try:
                self._originalSafeDict.pop(name)
            except KeyError:
                pass

    def resetSafe(self):
        """restores the safe dict with wich the evaluator was instantiated"""
        self.safe_dict = self._originalSafeDict.copy()

    def getSafe(self):
        """returns the currently whitelisted expressions"""
        return self.safe_dict


##-----------------------------------------------##
# A demo of use

if __name__ == '__main__':

    x = list(range(6))
    sev = SafeEvaluator()
    print("trying to evaluate a variable that has not been registered")
    try:
        # This will fail because 'x' is not registered in sev
        print(sev.safeEval('x+2'))
    except:
        print("failed!!")

    sev.addSafe({'x': x})  # After registering x, we can use it...
    f0 = 'x'
    f1 = 'sqrt(x)'
    f2 = 'pow(2,8)'
    f3 = 'ceil(array(x)/2.)'
    f4 = 'x[3]*2'
    # This is something we do not want to be evaluated
    f5 = 'open("/etc/passwd")'

    for f in [f0, f1, f2, f3, f4, f5]:
        print('Evaluating "%s":' % f)
        try:
            print(sev.eval(f))
        except:
            print('ERROR: %s cannot be evaluated' % f)

    import numpy
    vector = numpy.arange(6)
    # Another way of registering a variable is using the init method...
    sev2 = SafeEvaluator({'x': x, 'y': vector}, defaultSafe=False)
    print('x*y=', sev2.eval('x*y'))
    y = 0  # note that the registered variable is local to the evaluator!!
    # here, y still has the previously registered value instead of 0
    print('x*y=', sev2.eval('x*y'))



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Module containing base classes for using the TangoSchemeTest DS in tests"""

from builtins import object
import PyTango
from taurus.core.tango.starter import ProcessStarter
from taurus.test import getResourcePath

__all__ = ['TangoSchemeTestLauncher']

__docformat__ = 'restructuredtext'


class TangoSchemeTestLauncher(object):
    """A base class for TestCase classes wishing to start a TangoSchemeTest.
    Use it as a mixin class"""

    DEV_NAME = 'TangoSchemeTest/unittest/temp-1'  # the dev name

    @classmethod
    def setUpClass(cls):
        """ Create and run a TangoSchemeTest device server
        """
        # get path to DS and executable
        device = getResourcePath('taurus.core.tango.test.res',
                                 'TangoSchemeTest')
        # create starter for the device server
        cls._starter = ProcessStarter(device, 'TangoSchemeTest/unittest')
        # register
        cls._starter.addNewDevice(cls.DEV_NAME, klass='TangoSchemeTest')
        # start device server
        cls._starter.startDs()

    @classmethod
    def tearDownClass(cls):
        """ Stop the device server and undo changes to the database
        """
        d = PyTango.DeviceProxy(cls.DEV_NAME)
        d.Reset()
        cls._starter.stopDs(hard_kill=True)
        # remove server
        cls._starter.cleanDb(force=True)

    def tearDown(self):
        d = PyTango.DeviceProxy(self.DEV_NAME)
        d.Reset()

if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""[DEPRECATED since taurus v4]
This module contains the base class for a taurus attribute configuration"""

from builtins import object
from .taurusmodel import TaurusModel
from .util.log import taurus4_deprecation


__all__ = ["TaurusConfigurationProxy", "TaurusConfiguration"]

__docformat__ = "restructuredtext"


class TaurusConfigurationProxy(object):
    """
    TaurusAttribute has a reference to TaurusConfiguration and it should also have
    a reference to TaurusAttribute. To solve this cyclic dependency,
    TaurusConfiguration has a weak reference to TaurusAttribute. But then we must
    be sure that no other references to TaurusConfiguration exist so that
    no one tries to use it after its TaurusAttribute has disappeared.
    That's why to the outside world we don't give access to it directly
    but to objects of this new TaurusConfigurationProxy class.
    """
    @taurus4_deprecation(dbg_msg='Do not use this class')
    def __init__(self, parent):
        self.__parent = parent

    def __getattr__(self, name):
        return getattr(self.__parent._getRealConfig(), name)

    def getRealConfigClass(self):
        return self.__parent._getRealConfig().__class__


class TaurusConfiguration(TaurusModel):

    @taurus4_deprecation(alt='TaurusAttribute', dbg_msg='Do not use this class')
    def __init__(self, name, parent, storeCallback=None):
        pass



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
###########################################################################

"""
This Wizard provide functionality for creating from scratch a configuration
directory for a TaurusGUI based application.

The files in the configuration dir determine the default, permanent, pre-defined
contents of the GUI. While the user may add/remove more elements at run
time and those customizations will also be stored, this file defines what a
user will find when launching the GUI for the first time.
"""
from __future__ import print_function

from builtins import str

import os
import re
import sys
import shutil
import copy
import datetime
import glob
from lxml import etree

from taurus import tauruscustomsettings
from taurus.external.qt import Qt, compat
import taurus.qt.qtgui.panel
import taurus.qt.qtgui.taurusgui.paneldescriptionwizard
import taurus.qt.qtgui.input
from taurus.core.tango.tangovalidator import TangoDeviceNameValidator
from taurus.core.util.enumeration import Enumeration
from taurus.qt.qtgui.util import ExternalAppAction


__all__ = ["AppSettingsWizard", "ExternalAppEditor"]


class BooleanWidget(Qt.QWidget):
    """
    This class represents the simple boolean widget with two RadioButtons
    true and false. The default value of the widget is true.
    It change the value by using getValue and setValue methods
    """

    valueChangedSignal = Qt.pyqtSignal(bool, bool)

    def __init__(self, parent=None):
        Qt.QWidget.__init__(self, parent)
        self._formLayout = Qt.QHBoxLayout(self)
        self.trueButton = Qt.QRadioButton(self)
        self._formLayout.addWidget(self.trueButton)
        self.falseButton = Qt.QRadioButton(self)
        self._formLayout.addWidget(self.falseButton)
        self.trueButton.setText("Yes")
        self.falseButton.setText("No")
        self.trueButton.clicked.connect(self.valueChanged)
        self.falseButton.clicked.connect(self.valueChanged)
        self.setValue(self.getDefaultValue())

    def valueChanged(self):
        if not (self.trueButton.isChecked() == self._actualValue):
            self.valueChangedSignal.emit(self._actualValue, not self._actualValue)
        self._actualValue = self.trueButton.isChecked()

    def setValue(self, value):
        if value is None:
            value = self.getDefaultValue()
        self.trueButton.setChecked(value)
        self.falseButton.setChecked(not value)
        self._actualValue = value

    def getValue(self):
        return self.trueButton.isChecked()

    @classmethod
    def getDefaultValue(self):
        return False


class BasePage(Qt.QWizardPage):
    """
        This class represents the base page for all of the pages in the wizard
    """

    completeChanged = Qt.pyqtSignal()

    def __init__(self, parent=None):
        Qt.QWizardPage.__init__(self, parent)
        self._item_funcs = {}
        self._layout = Qt.QGridLayout()
        self.setLayout(self._layout)
        self._setupUI()

    def initializePage(self):
        Qt.QWizardPage.initializePage(self)
        self.checkData()

    def fromXml(self, xml):
        """
        :param xml: (etree.Element) root node
        """
        pass

    def _setupUI(self):
        pass

    def checkData(self):
        self._valid = True
        self.completeChanged.emit()

    def isComplete(self):
        return self._valid

    def _markRed(self, label):
        """
            Set the color of the given label to red
        """
        palette = label.palette()
        palette.setBrush(Qt.QPalette.WindowText, Qt.Qt.red)
        label.update()

    def _markBlack(self, label):
        """
            Set the color of the given label to black
        """
        palette = label.palette()
        palette.setBrush(Qt.QPalette.WindowText, Qt.Qt.black)
        label.update()

    def setStatusLabelPalette(self, label):
        """
            Set the label look as as status label
        """
        label.setAutoFillBackground(True)
        palette = label.palette()
        gradient = Qt.QLinearGradient(0, 0, 0, 15)
        gradient.setColorAt(0.0, Qt.QColor.fromRgb(60, 150, 255))
        gradient.setColorAt(0.5, Qt.QColor.fromRgb(0, 85, 227))
        gradient.setColorAt(1.0, Qt.QColor.fromRgb(60, 150, 255))
        gradient.setSpread(Qt.QGradient.RepeatSpread)
        palette.setBrush(Qt.QPalette.Window, Qt.QBrush(gradient))
        palette.setBrush(Qt.QPalette.WindowText, Qt.Qt.white)

    def __setitem__(self, name, value):
        self._item_funcs[name] = value

    def __getitem__(self, name):
        return self._item_funcs[name]

    def setNextPageId(self, id):
        self._nextPageId = id

    def nextId(self):
        return self._nextPageId


class IntroPage(BasePage):
    """
        Introduction page
    """

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)

    def _setupUI(self):
        self.setTitle('Introduction')
        self.setPixmap(Qt.QWizard.WatermarkPixmap, Qt.QIcon.fromTheme(
            "document-properties").pixmap(120, 120))
        label = Qt.QLabel(self.getIntroText())
        label.setWordWrap(True)
        self._layout.addWidget(label, 0, 0)
        self._spacerItem1 = Qt.QSpacerItem(
            10, 200, Qt.QSizePolicy.Minimum, Qt.QSizePolicy.Fixed)
        self._layout.addItem(self._spacerItem1, 1, 0)
        self.setLayout(self._layout)

    def getIntroText(self):
        text = 'This wizard will guide you through the process of creating a ' + \
               'GUI based on TaurusGUI.\n' + \
               'TaurusGui-based applications are very customizable. The user can ' + \
               'add/remove elements at run time and store those customizations. So ' + \
               'with this wizard you will define just the default contents of the GUI.'
        return text

    def setNextPageId(self, id):
        self._nextPageId = id


class ProjectPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)
        self.setTitle('Project')
        self.setSubTitle(
            'Choose a location for the application files (i.e., the "project directory")')
        self.__setitem__('projectDir', self._getProjectDir)

    def _setupUI(self):
        BasePage._setupUI(self)
        self._projectDirLabel = Qt.QLabel("Project Directory:")
        self._projectDirLE = Qt.QLineEdit(Qt.QDir.homePath())
        self._projectDirLE.setMinimumSize(150, 30)
        self._projectDirLE.setToolTip(
            'This directory will be used to store all files needed by the application.')
        self._projectDirBT = Qt.QPushButton(Qt.QIcon.fromTheme(
            "document-properties"), '...')
        self._layout.addWidget(self._projectDirLabel, 1, 0)
        self._layout.addWidget(self._projectDirLE, 1, 1)
        self._layout.addWidget(self._projectDirBT, 1, 2)

        self._projectDirBT.clicked.connect(self.onSelectDir)

    def onSelectDir(self):
        dirname = str(Qt.QFileDialog.getExistingDirectory(
            self, 'Choose the project directory', self._projectDirLE.text()))
        if not dirname:
            return
        self._projectDirLE.setText(dirname)

    def validatePage(self):
        dirname = str(self._projectDirLE.text())

        if not os.path.exists(dirname):
            try:
                os.makedirs(dirname)
            except Exception as e:
                Qt.QMessageBox.warning(self, 'Error creating project directory',
                                       'Could not create the project directory.\nReason:%s' % repr(
                                           e),
                                       Qt.QMessageBox.Cancel)
                return False

        configs_found = glob.glob(os.path.join(dirname, "tgconf_*",
                                  self.wizard().getXmlConfigFileName()))
        # fname = os.path.join(dirname, self.wizard().getXmlConfigFileName())
        if len(configs_found) == 1:
            fname = configs_found[0]
            option = Qt.QMessageBox.question(self, 'Overwrite project?',
                                             'The "%s" file already exists in the project directory.\n Do you want to edit the existing project?' % (
                                                 os.path.basename(fname)),
                                             Qt.QMessageBox.Yes | Qt.QMessageBox.Cancel)
            if option == Qt.QMessageBox.Yes:
                try:
                    self.wizard().loadXml(fname)
                except Exception as e:
                    Qt.QMessageBox.warning(self, 'Error loading project configuration',
                                           'Could not load the existing configuration.\nReason:%s' % repr(
                                               e),
                                           Qt.QMessageBox.Cancel)
                    return False
            else:
                return False
        elif len(os.listdir(dirname)):
            option = Qt.QMessageBox.question(self, 'Non empty project dir',
                                             'The project directory ("%s") is not empty.\nAre you sure you want to use it?' % (
                                                 os.path.basename(dirname)),
                                             Qt.QMessageBox.Yes | Qt.QMessageBox.No)
            if option != Qt.QMessageBox.Yes:
                return False
        # if all went ok...
        return True

    def _getProjectDir(self):
        return str(self._projectDirLE.text())


class GeneralSettings(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)
        self.setTitle('General settings')

    def initializePage(self):
        BasePage.initializePage(self)
        self.wizard().__setitem__("guiName", self._getGUIName)
        self.wizard().__setitem__("organizationName", self._getOrganizationName)

    def fromXml(self, xml):
        self._guiNameLineEdit.setText(
            AppSettingsWizard.getValueFromNode(xml, "GUI_NAME", ''))
        self._organizationCombo.setEditText(
            AppSettingsWizard.getValueFromNode(xml, "ORGANIZATION", default='Taurus'))

    def _getGUIName(self):
        return str(self._guiNameLineEdit.text())

    def _getOrganizationName(self):
        if len(self._organizationCombo.currentText()) > 0:
            return str(self._organizationCombo.currentText())
        else:
            return None

    def _setupUI(self):
        BasePage._setupUI(self)
        self._guiNameLabel = Qt.QLabel("GUI name:")
        font = Qt.QFont()  # set bigger font
        font.setPointSize(14)

        self._label = Qt.QLabel()
        self._layout.addWidget(self._label, 0, 0, 1, 2, Qt.Qt.AlignRight)
        self._guiNameLineEdit = Qt.QLineEdit()
        self._guiNameLineEdit.setFont(font)
        self._guiNameLineEdit.setMinimumSize(150, 30)
        self._layout.addWidget(self._guiNameLabel, 1, 0,
                               1, 1, Qt.Qt.AlignRight)
        self._layout.addWidget(self._guiNameLineEdit, 1,
                               1, 1, 1, Qt.Qt.AlignRight)
        self._organizationNameLabel = Qt.QLabel("Organization name:")
        self._organizationCombo = Qt.QComboBox()
        self._organizationCombo.addItems(self._getOrganizationNames())
        self._organizationCombo.setMinimumSize(150, 25)
        self._organizationCombo.setEditable(True)
        self._layout.addWidget(self._organizationNameLabel,
                               2, 0, 1, 1, Qt.Qt.AlignRight)
        self._layout.addWidget(self._organizationCombo,
                               2, 1, 1, 1, Qt.Qt.AlignRight)

        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel()
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 3)

        self._guiNameLineEdit.textChanged.connect(self.checkData)
        self._organizationCombo.editTextChanged.connect(self.checkData)
        self._organizationCombo.currentIndexChanged.connect(self.checkData)

    def _getOrganizationNames(self):
        return ["TAURUS", "ALBA", "DESY", "Elettra", "ESRF", "MAX-lab", "SOLEIL", "XFEL"]

    def checkData(self):
        self._valid = True
        if not len(self._guiNameLineEdit.text()):
            self._valid = False
            self._markRed(self._guiNameLabel)
        else:
            self._markBlack(self._guiNameLabel)

        self.completeChanged.emit()

        if not self._valid:
            self._setStatus("Please type the name of the GUI")
        else:
            self._setStatus("Press next button to continue")

    def _setStatus(self, text):
        self._status_label.setText(text)


class CustomLogoPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)
        self._customLogoDefaultPath = getattr(tauruscustomsettings,
                                              "ORGANIZATION_LOGO",
                                              "logos:taurus.png")
        self._customLogoPath = self._customLogoDefaultPath

    def initializePage(self):
        BasePage.initializePage(self)
        self.wizard().__setitem__("customLogo", self._getCustomLogo)
        self._changeImage()

    def _setupUI(self):
        BasePage._setupUI(self)
        self.setTitle('Custom logo')
        self._label = Qt.QLabel(
            "\nIf you want to have a custom logo inside your application panel, please select the image file. \n")
        self._label.setWordWrap(True)
        self._layout.addWidget(self._label, 0, 0, 1, 4)
        self._customLogoLabel = Qt.QLabel("Custom logo:")
        self._customLogoLineEdit = Qt.QLineEdit()
        self._customLogoLineEdit.setMinimumSize(250, 25)
        self._customLogoLineEdit.setReadOnly(False)
        self._customLogoButton = Qt.QPushButton()
        self._customLogoButton.setToolTip("Browse...")
        self._customLogoButton.setIcon(Qt.QIcon.fromTheme("folder-open"))
        self._customLogoButton.setMaximumSize(80, 25)
        self._spacerItem1 = Qt.QSpacerItem(
            30, 30, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Fixed)
        self._customLogo = Qt.QLabel(self)
        self._customLogo.setAlignment(Qt.Qt.AlignCenter)
        self._customLogo.setMinimumSize(120, 120)
        self._customLogoDefaultButton = Qt.QPushButton()
        self._customLogoDefaultButton.setToolTip("Default")
        self._customLogoDefaultButton.setMaximumSize(80, 25)
        self._customLogoDefaultButton.setIcon(Qt.QIcon("actions:edit-undo.svg"))
        self._customLogoRemoveButton = Qt.QPushButton()
        self._customLogoRemoveButton.setToolTip("Remove")
        self._customLogoRemoveButton.setMaximumSize(80, 25)
        self._customLogoRemoveButton.setIcon(
            Qt.QIcon("emblems:emblem-unreadable.svg"))
        self._spacerItem2 = Qt.QSpacerItem(
            30, 30, Qt.QSizePolicy.Expanding, Qt.QSizePolicy.Fixed)

        self._layout.addWidget(self._customLogoLabel, 2, 0, Qt.Qt.AlignRight)
        self._layout.addWidget(self._customLogoLineEdit,
                               2, 1, Qt.Qt.AlignRight)
        self._layout.addWidget(self._customLogoButton, 2, 2, Qt.Qt.AlignLeft)
        self._layout.addWidget(
            self._customLogoDefaultButton, 2, 3, Qt.Qt.AlignLeft)
        self._layout.addWidget(
            self._customLogoRemoveButton, 2, 4, Qt.Qt.AlignLeft)
        self._layout.addItem(self._spacerItem2, 2, 5)
        self._layout.addItem(self._spacerItem1, 3, 2)
        self._layout.addWidget(self._customLogo, 4, 1,
                               1, 1, Qt.Qt.AlignHCenter)

        self._customLogoButton.clicked.connect(self._selectImage)
        self._customLogoDefaultButton.clicked.connect(self._setDefaultImage)
        self._customLogoRemoveButton.clicked.connect(self._removeImage)
        self._customLogoLineEdit.textChanged.connect(self._changeImage)

        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel("Press next button to continue")
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 6)
        self._setNoImage()

    def fromXml(self, xml):
        customLogo = AppSettingsWizard.getValueFromNode(
            xml, "CUSTOM_LOGO", None)
        if customLogo is None:
            self._setDefaultImage()
        else:
            self._customLogoLineEdit.setText(customLogo)

    def _setDefaultImage(self):
        self._customLogoLineEdit.setText(self._customLogoDefaultPath)

    def _setNoImage(self):
        self._customLogo.setPixmap(
            Qt.QIcon.fromTheme("image-missing").pixmap(50, 50))
        self._customLogoPath = None
        self._customLogoRemoveButton.hide()

    def _removeImage(self):
        self._customLogoLineEdit.setText("")
        self._setNoImage()

    def _getCustomLogo(self):
        if (self._customLogoPath is not None):
            return str(self._customLogoPath)
        else:
            return None

    def _selectImage(self):
        fileName, _ = compat.getOpenFileName(
            self, self.tr("Open File"), Qt.QDir.homePath(),
            self.tr("Images (*.png *.xpm *.jpg *.jpeg *.svg)")
        )
        self._customLogoLineEdit.setText(fileName)
        self._changeImage()

    def _changeImage(self):
        fileName = str(self._customLogoLineEdit.text())
        if len(fileName):
            if (os.path.exists(fileName)):
                image = Qt.QImage()
                if image.load(fileName):
                    self._setImage(image)
                    self._customLogoPath = fileName
                    self._setStatus("Press next button to continue")
                    self._customLogoRemoveButton.show()
                else:
                    self._setNoImage()
                    self._setStatus("The file is invalid")
            else:
                self._setNoImage()
                self._setStatus("The file does not exist")
        else:
            self._setNoImage()
            self._setStatus("No image")

    def _setImage(self, image):
        if type(image) == Qt.QPixmap:
            self._customLogo.setPixmap(
                image.scaled(60, 200, Qt.Qt.KeepAspectRatio))
        elif type(image) == Qt.QImage:
            self._customLogo.setPixmap(Qt.QPixmap().fromImage(
                image).scaled(60, 200, Qt.Qt.KeepAspectRatio))
        else:
            self._customLogo.setPixmap(
                Qt.QPixmap("image-missing").scaled(50, 50))
            self._customLogoPath = None

    def _setStatus(self, text):
        self._status_label.setText(text)


class SynopticPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)
        self._synoptics = []

    def fromXml(self, xml):
        self._synoptics = []
        synopticNodes = AppSettingsWizard.getArrayFromNode(
            xml, "SYNOPTIC", default=[])
        for child in synopticNodes:
            if child.get("str") is not None and len(child.get("str")):
                self._synoptics.append(child.get("str"))

    def initializePage(self):
        BasePage.initializePage(self)
        self.wizard().__setitem__("synoptics", self._getSynoptics)
        self._refreshSynopticList()

    def _setupUI(self):
        BasePage._setupUI(self)
        self.setTitle('Synoptics')
        self._label = Qt.QLabel(
            "If you want to add one or more synoptic panels (graphical views of instruments) select the corresponding JDRAW files here\n")
        self._label.setWordWrap(True)
        self._layout.addWidget(self._label, 0, 0)
        self.setLayout(self._layout)
        self._synopticGroupBox = Qt.QGroupBox()
        self._synopticGroupBox.setCheckable(False)
        self._synopticGroupBox.setAlignment(Qt.Qt.AlignLeft)
        self._synopticGroupBox.setStyleSheet(
            " QGroupBox::title {  subcontrol-position: top left; padding: 5 5px; }")
        self._layout.addWidget(self._synopticGroupBox, 2, 0, 1, 1)
        self._horizontalLayout = Qt.QHBoxLayout(self._synopticGroupBox)
        self._synopticList = Qt.QListWidget(self._synopticGroupBox)
        self._horizontalLayout.addWidget(self._synopticList)
        self._verticalLayout = Qt.QVBoxLayout()
        self._addButton = Qt.QPushButton("Add Synoptic")
        self._addButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._addButton)
        self._removeButton = Qt.QPushButton("Remove Synoptic")
        self._removeButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._removeButton)
        self._upButton = Qt.QPushButton("Move Up")
        self._upButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._upButton)
        self._downButton = Qt.QPushButton("Move Down")
        self._downButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._downButton)
        self._horizontalLayout.addLayout(self._verticalLayout)
        self._addButton.setIcon(Qt.QIcon.fromTheme("list-add"))
        self._removeButton.setIcon(Qt.QIcon.fromTheme("list-remove"))
        self._upButton.setIcon(Qt.QIcon.fromTheme("go-up"))
        self._downButton.setIcon(Qt.QIcon.fromTheme("go-down"))
        self._addButton.clicked.connect(self._addSynoptic)
        self._removeButton.clicked.connect(self._removeSynoptic)
        self._upButton.clicked.connect(self._moveUp)
        self._downButton.clicked.connect(self._moveDown)
        #self._synopticList.itemDoubleClicked.connect(self._editSynoptic)
        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel("Press next button to continue")
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 1)

    def _addSynoptic(self):
        pdir = self.wizard().__getitem__('projectDir')
        fileNames, _ = compat.getOpenFileNames(
            self, self.tr("Open File"), pdir,
            self.tr("JDW (*.jdw );; All files (*)")
        )
        for fileName in fileNames:
            fileName = str(fileName)
            if fileName not in self._synoptics:
                self._synoptics.append(fileName)
        self._refreshSynopticList()

    def _editSynoptic(self):
        # edit
        self._refreshSynopticList()

    def _removeSynoptic(self):
        if len(self._synopticList.selectedIndexes()) > 0:
            self._synoptic_id = self._synopticList.selectedIndexes()[0].row()
            self._synoptics.remove(self._synoptics[self._synoptic_id])
            self._refreshSynopticList()

    def _moveUp(self):
        if len(self._synopticList.selectedIndexes()) > 0:
            self._synoptic_id = self._synopticList.selectedIndexes()[0].row()
            if self._synoptic_id > 0:
                tmp = self._synoptics[self._synoptic_id]
                self._synoptics[self._synoptic_id] = self._synoptics[
                    self._synoptic_id - 1]
                self._synoptics[self._synoptic_id - 1] = tmp
                self._refreshSynopticList()
                self._synopticList.setCurrentIndex(self._synopticList.indexFromItem(
                    self._synopticList.item(self._synoptic_id - 1)))

    def _moveDown(self):
        if len(self._synopticList.selectedIndexes()) > 0:
            self._synoptic_id = self._synopticList.selectedIndexes()[0].row()
            if self._synoptic_id < self._synopticList.count() - 1:
                tmp = self._synoptics[self._synoptic_id]
                self._synoptics[self._synoptic_id] = self._synoptics[
                    self._synoptic_id + 1]
                self._synoptics[self._synoptic_id + 1] = tmp
                self._refreshSynopticList()
                self._synopticList.setCurrentIndex(self._synopticList.indexFromItem(
                    self._synopticList.item(self._synoptic_id + 1)))

    def _refreshSynopticList(self):
        self._synopticList.clear()
        for name in self._synoptics:
            self._synopticList.addItem(name)

    def _getSynoptics(self):
        if len(self._synoptics) <= 0:
            return None
        else:
            return self._synoptics

    def checkData(self):
        BasePage.checkData(self)
        self._valid = True

    def _setStatus(self, text):
        self._status_label.setText(text)


class MacroServerInfoPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)

    def initializePage(self):
        BasePage.initializePage(self)
        self._label.setText("\n <b>%s</b> can communicate with a Sardana's Macro Server and Pool.\nYou can enable and configure them here:\n" %
                            self.wizard().__getitem__("guiName"))
        self.wizard().__setitem__("macroServerName", self._getMacroServerName)
        self.wizard().__setitem__("doorName", self._getDoorName)

    def _setupUI(self):
        BasePage._setupUI(self)
        self.setTitle('Macro Server Info')
        self._label = Qt.QLabel()
        self._label.setWordWrap(True)
        self._macroGroupBox = Qt.QGroupBox()
        self._macroGroupBox.setTitle("Enable Sardana communication")
        self._macroGroupBox.setCheckable(True)
        self._macroGroupBox.setChecked(False)
        self._macroGroupBox.setAlignment(Qt.Qt.AlignLeft)
        self._macroGroupBox.setStyleSheet(
            " QGroupBox::title {  subcontrol-position: top left; padding: 5 5px; }")
        self._horizontalLayout = Qt.QHBoxLayout(self._macroGroupBox)
        from sardana.taurus.qt.qtgui.extra_macroexecutor.common import \
                TaurusMacroConfigurationDialog
        self._confWidget = TaurusMacroConfigurationDialog(self)
        self._confWidget.setWindowFlags(Qt.Qt.Widget)
        self._confWidget.setModal(False)
        self._confWidget.setVisible(True)
        self._confWidget.buttonBox.setVisible(False)
        self._horizontalLayout.addWidget(self._confWidget)

        self._layout.addWidget(self._label, 0, 0, 1, 1)
        self._layout.addWidget(self._macroGroupBox, 1, 0, 1, 1)

        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel("Press next button to continue")
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 1)

        self._confWidget.macroServerComboBox.currentIndexChanged.connect(
            self.checkData)
        self._confWidget.doorComboBox.currentIndexChanged.connect(
            self.checkData)
        self._macroGroupBox.toggled.connect(self.checkData)

    def fromXml(self, xml):
        macroserverName = AppSettingsWizard.getValueFromNode(
            xml, "MACROSERVER_NAME", default="")
        doorName = AppSettingsWizard.getValueFromNode(
            xml, "DOOR_NAME", default="")
        macroEditorsPath = AppSettingsWizard.getValueFromNode(
            xml, "MACROEDITORS_PATH", default="")

        id = self._confWidget.macroServerComboBox.findText(
            macroserverName, Qt.Qt.MatchExactly)
        if id >= 0:
            self._confWidget.macroServerComboBox.setCurrentIndex(id)
            self._macroGroupBox.setChecked(True)
        else:
            self._macroGroupBox.setChecked(False)
            return

        id = self._confWidget.doorComboBox.findText(
            doorName, Qt.Qt.MatchExactly)
        if id >= 0:
            self._confWidget.doorComboBox.setCurrentIndex(id)

    def checkData(self):
        BasePage.checkData(self)
        if (self._macroGroupBox.isChecked()) and len(self._confWidget.macroServerComboBox.currentText()):
            self.setNextPageId(self.wizard().currentId() + 1)
        else:
            self.setNextPageId(self.wizard().currentId() + 2)

    def _getMacroServerName(self):
        if (self._macroGroupBox.isChecked()) and len(self._confWidget.macroServerComboBox.currentText()):
            ms_name = str(self._confWidget.macroServerComboBox.currentText())
            complete_name, _, _ = TangoDeviceNameValidator().getNames(ms_name)
            return complete_name
        else:
            return None

    def _getDoorName(self):
        if (self._macroGroupBox.isChecked()) and len(self._confWidget.macroServerComboBox.currentText()):
            door_name = str(self._confWidget.doorComboBox.currentText())
            complete_name, _, _ = TangoDeviceNameValidator().getNames(
                door_name)
            return complete_name
        else:
            return None

    def _setStatus(self, text):
        self._status_label.setText(text)


class InstrumentsPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)

    def initializePage(self):
        BasePage.initializePage(self)
        self.wizard().__setitem__("instruments", self._getInstruments)
        self._label.setText("<b>%s</b> can use instrument information stored in the Sardana's Pool to create instrument panels." %
                            self.wizard().__getitem__("guiName"))

    def _setupUI(self):
        BasePage._setupUI(self)
        self.setTitle('Instruments from Pool:')
        self._label = Qt.QLabel()
        self._label.setWordWrap(True)
        self._layout.addWidget(self._label, 0, 0, 1, 3)

        self._instrumentsLabel = Qt.QLabel("Generate panels from Pool Info?")
        self._intstrumentsBoolean = BooleanWidget()
        self._intstrumentsBoolean.setMinimumSize(150, 25)
        self._layout.addWidget(self._instrumentsLabel, 5,
                               0, 1, 1, Qt.Qt.AlignRight)
        self._layout.addWidget(self._intstrumentsBoolean,
                               5, 1, 1, 1, Qt.Qt.AlignRight)

        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel("Press next button to continue")
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 3)

    def fromXml(self, xml):
        instruments = AppSettingsWizard.getValueFromNode(
            xml, "INSTRUMENTS_FROM_POOL", default="False")
        if str(instruments).lower() == "true":
            self._intstrumentsBoolean.setValue(True)
        else:
            self._intstrumentsBoolean.setValue(False)

    def _getInstruments(self):
        return str(self._intstrumentsBoolean.getValue())

    def checkData(self):
        self._valid = True

    def _setStatus(self, text):
        self._status_label.setText(text)


class PanelsPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)
        self._panels = []

    def initializePage(self):
        BasePage.initializePage(self)
        self.wizard().__setitem__("panels", self._getPanels)
        self._refreshPanelList()

    def _setupUI(self):
        BasePage._setupUI(self)
        self.setTitle('Panels editor')
        self._label = Qt.QLabel(
            "If you want extra panels add them to this list\n")
        self._layout.addWidget(self._label, 0, 0)
        self.setLayout(self._layout)
        self._panelGroupBox = Qt.QGroupBox()
        self._panelGroupBox.setCheckable(False)
        self._panelGroupBox.setAlignment(Qt.Qt.AlignLeft)
        self._panelGroupBox.setStyleSheet(
            " QGroupBox::title {  subcontrol-position: top left; padding: 5 5px; }")
        self._layout.addWidget(self._panelGroupBox, 2, 0, 1, 1)
        self._horizontalLayout = Qt.QHBoxLayout(self._panelGroupBox)
        self._panelList = Qt.QListWidget(self._panelGroupBox)
        self._horizontalLayout.addWidget(self._panelList)
        self._verticalLayout = Qt.QVBoxLayout()
        self._addButton = Qt.QPushButton("Add Panel")
        self._addButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._addButton)
        self._removeButton = Qt.QPushButton("Remove Panel")
        self._removeButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._removeButton)
        self._upButton = Qt.QPushButton("Move Up")
        self._upButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._upButton)
        self._downButton = Qt.QPushButton("Move Down")
        self._downButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._downButton)
        self._horizontalLayout.addLayout(self._verticalLayout)
        self._addButton.setIcon(Qt.QIcon.fromTheme("list-add"))
        self._removeButton.setIcon(Qt.QIcon.fromTheme("list-remove"))
        self._upButton.setIcon(Qt.QIcon.fromTheme("go-up"))
        self._downButton.setIcon(Qt.QIcon.fromTheme("go-down"))
        self._addButton.clicked.connect(self._addPanel)
        self._removeButton.clicked.connect(self._removePanel)
        self._upButton.clicked.connect(self._moveUp)
        self._downButton.clicked.connect(self._moveDown)
        self._panelList.itemDoubleClicked.connect(self._editPanel)
        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel("Press next button to continue")
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 1)

    def fromXml(self, xml):
        self._panels = []
        panelNodes = AppSettingsWizard.getArrayFromNode(
            xml, "PanelDescriptions", default=[])
        for child in panelNodes:
            name = AppSettingsWizard.getValueFromNode(
                child, "name", default=None)
            if name:
                self._panels.append((name, etree.tostring(child)))

    def _addPanel(self):
        paneldesc, ok = taurus.qt.qtgui.taurusgui.paneldescriptionwizard.PanelDescriptionWizard.getDialog(
            self)
        if ok:
            w = paneldesc.getWidget()
            self._panels.append((paneldesc.name, paneldesc.toXml()))

        self._refreshPanelList()

    def _editPanel(self):
        # edit
        self._refreshPanelList()

    def _removePanel(self):
        if len(self._panelList.selectedIndexes()) > 0:
            self._panel_id = self._panelList.selectedIndexes()[0].row()
            self._panels.remove(self._panels[self._panel_id])
            self._refreshPanelList()

    def _moveUp(self):
        if len(self._panelList.selectedIndexes()) > 0:
            self._panel_id = self._panelList.selectedIndexes()[0].row()
            if self._panel_id > 0:
                tmp = self._panels[self._panel_id]
                self._panels[self._panel_id] = self._panels[self._panel_id - 1]
                self._panels[self._panel_id - 1] = tmp
                self._refreshPanelList()
                self._panelList.setCurrentIndex(self._panelList.indexFromItem(
                    self._panelList.item(self._panel_id - 1)))

    def _moveDown(self):
        if len(self._panelList.selectedIndexes()) > 0:
            self._panel_id = self._panelList.selectedIndexes()[0].row()
            if self._panel_id < self._panelList.count() - 1:
                tmp = self._panels[self._panel_id]
                self._panels[self._panel_id] = self._panels[self._panel_id + 1]
                self._panels[self._panel_id + 1] = tmp
                self._refreshPanelList()
                self._panelList.setCurrentIndex(self._panelList.indexFromItem(
                    self._panelList.item(self._panel_id + 1)))

    def _refreshPanelList(self):
        self._panelList.clear()
        for panel in self._panels:
            name, xml = panel
            self._panelList.addItem(name)

    def _getPanels(self):
        if len(self._panels) <= 0:
            return None
        else:
            return self._panels

    def checkData(self):
        BasePage.checkData(self)
        self._valid = True

    def _setStatus(self, text):
        self._status_label.setText(text)


class ExternalAppEditor(Qt.QDialog):
    '''
    A dialog for configuring an external appaction for a TaurusMainWindow.
    '''
    #@todo: this class should be made more generic (e.g. provide a getter for an ExternalAppAction) and then moved elsewhere

    def __init__(self, parent=None):
        Qt.QDialog.__init__(self, parent)
        self.setModal(True)
        self.setWindowTitle('External Application Editor')

        self._dlgBox = Qt.QDialogButtonBox(
            Qt.QDialogButtonBox.Ok | Qt.QDialogButtonBox.Cancel)

        self._layout = Qt.QVBoxLayout()
        self._layout1 = Qt.QGridLayout()
        self._layout2 = Qt.QHBoxLayout()
        self._layout.addLayout(self._layout1)
        self._layout.addLayout(self._layout2)
        self._layout.addWidget(self._dlgBox)
        self.setLayout(self._layout)

        self._icon = None
        self._label = Qt.QLabel(
            "\n On this page you can define an external application. \n")
        self._label.setWordWrap(True)
        self._layout1.addWidget(self._label, 0, 0, 1, 4)
        self._execFileLabel = Qt.QLabel("Command:")
        self._execFileLineEdit = Qt.QLineEdit()
        self._execFileLineEdit.setMinimumSize(150, 25)
        # self._execFileLineEdit.setReadOnly(True)
        self._execFileButton = Qt.QPushButton()
        self._execFileButton.setIcon(Qt.QIcon.fromTheme("folder-open"))
        self._execFileButton.setToolTip("Browse...")
        self._execFileButton.setMaximumSize(80, 25)
        self._layout1.addWidget(self._execFileLabel, 2, 0, Qt.Qt.AlignRight)
        self._layout1.addWidget(self._execFileLineEdit, 2, 1, Qt.Qt.AlignRight)
        self._layout1.addWidget(self._execFileButton, 2, 2, Qt.Qt.AlignLeft)
        self._paramsLabel = Qt.QLabel("Parameters:")
        self._paramsLineEdit = Qt.QLineEdit()
        self._paramsLineEdit.setMinimumSize(150, 25)
        self._layout1.addWidget(self._paramsLabel, 3, 0, Qt.Qt.AlignRight)
        self._layout1.addWidget(self._paramsLineEdit, 3, 1, Qt.Qt.AlignRight)
        self._textLabel = Qt.QLabel("Text:")
        self._textLineEdit = Qt.QLineEdit()
        self._textLineEdit.setMinimumSize(150, 25)
        self._layout1.addWidget(self._textLabel, 4, 0, Qt.Qt.AlignRight)
        self._layout1.addWidget(self._textLineEdit, 4, 1, Qt.Qt.AlignRight)

        self._iconLabel = Qt.QLabel("Icon:")
        self._iconLogo = Qt.QPushButton()
        self._iconLogo.setIcon(Qt.QIcon("status:image-missing.svg"))
        self._iconLogo.setIconSize(Qt.QSize(60, 60))
        self._layout1.addWidget(self._iconLabel, 5, 0, Qt.Qt.AlignRight)
        self._layout1.addWidget(self._iconLogo, 5, 1, Qt.Qt.AlignCenter)
        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout1.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)

        # connections
        self._execFileButton.clicked.connect(self._selectExecFile)
        self._execFileLineEdit.textChanged.connect(self._setDefaultText)
        self._iconLogo.clicked.connect(self._selectIcon)
        self._dlgBox.accepted.connect(self.accept)
        self._dlgBox.rejected.connect(self.reject)
        self.checkData()
        self._setIcon(ExternalAppAction.DEFAULT_ICON_NAME)

    def checkData(self):
        if len(self._execFileLineEdit.text()) > 0:
            self._dlgBox.button(Qt.QDialogButtonBox.Ok).setEnabled(True)
        else:
            self._dlgBox.button(Qt.QDialogButtonBox.Ok).setEnabled(False)

    def _setDefaultText(self):
        fileName = self._execFileLineEdit.text().split('/')[-1]
        index = str(fileName).rfind(".")
        if (index > 0):
            self._textLineEdit.setText(str(fileName)[0:index])
        else:
            self._textLineEdit.setText(fileName)
        self.checkData()

    def _selectExecFile(self):
        filePath, _ = compat.getOpenFileName(
            self, self.tr("Open File"),
            Qt.QDir.homePath(), self.tr("All files (*)")
        )
        if len(filePath):
            self._execFileLineEdit.setText(filePath)
            self._setDefaultText()

    def _getExecFile(self):
        return str(self._execFileLineEdit.text())

    def _selectIcon(self):
        from taurus.qt.qtgui.icon import QIconCatalog
        catalog = QIconCatalog()
        dlg = Qt.QDialog(self)
        dlg.setLayout(Qt.QVBoxLayout())
        dlg.layout().addWidget(catalog)
        dlg.setWindowTitle('Icon Catalog')
        catalog.iconSelected.connect(self._setIcon)
        catalog.iconSelected.connect(dlg.accept)
        dlg.exec_()

    def _setIcon(self, name):
        self._iconLogo.setIcon(Qt.QIcon(name))
        self._iconLogo.setIconSize(Qt.QSize(60, 60))
        self._iconLogo.setText("")
        self._icon = name

    def _getParams(self):
        return str(self._paramsLineEdit.text())
        # return str(self._paramsLineEdit.text()).split()

    def _getText(self):
        return str(self._textLineEdit.text())

    def _getIcon(self):
        return str(self._icon)

    def _toXml(self):
        root = etree.Element("ExternalApp")
        command = etree.SubElement(root, "command")
        command.text = self._getExecFile()
        params = etree.SubElement(root, "params")
        params.text = self._getParams()
        text = etree.SubElement(root, "text")
        text.text = self._getText()
        icon = etree.SubElement(root, "icon")
        icon.text = self._getIcon()

        return etree.tostring(root)

    @staticmethod
    def getDialog():
        dlg = ExternalAppEditor()
        dlg.exec_()
        return dlg._getExecFile(), dlg._toXml(), (dlg.result() == dlg.Accepted)


class ExternalAppPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)
        self._externalApps = []

    def initializePage(self):
        BasePage.initializePage(self)
        self.wizard().__setitem__("externalApps", self._getExternalApps)
        self._refreshApplicationList()

    def _setupUI(self):
        BasePage._setupUI(self)
        self.setTitle('External Applications')
        self._label = Qt.QLabel(
            "The GUI may include shortcuts to external applications. You can add them now.\n")
        self._layout.addWidget(self._label, 0, 0)
        self.setLayout(self._layout)
        self._externalAppGroupBox = Qt.QGroupBox()
        self._externalAppGroupBox.setCheckable(False)
        self._externalAppGroupBox.setAlignment(Qt.Qt.AlignLeft)
        self._externalAppGroupBox.setStyleSheet(
            " QGroupBox::title {  subcontrol-position: top left; padding: 5 5px; }")
        self._layout.addWidget(self._externalAppGroupBox, 2, 0, 1, 1)
        self._horizontalLayout = Qt.QHBoxLayout(self._externalAppGroupBox)
        self._externalAppList = Qt.QListWidget(self._externalAppGroupBox)
        self._horizontalLayout.addWidget(self._externalAppList)
        self._verticalLayout = Qt.QVBoxLayout()
        self._addButton = Qt.QPushButton("Add Application")
        self._addButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._addButton)
        self._removeButton = Qt.QPushButton("Remove Application")
        self._removeButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._removeButton)
        self._upButton = Qt.QPushButton("Move Up")
        self._upButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._upButton)
        self._downButton = Qt.QPushButton("Move Down")
        self._downButton.setStyleSheet("text-align: left;")
        self._verticalLayout.addWidget(self._downButton)
        self._horizontalLayout.addLayout(self._verticalLayout)
        self._addButton.setIcon(Qt.QIcon.fromTheme("list-add"))
        self._removeButton.setIcon(Qt.QIcon.fromTheme("list-remove"))
        self._upButton.setIcon(Qt.QIcon.fromTheme("go-up"))
        self._downButton.setIcon(Qt.QIcon.fromTheme("go-down"))
        self._addButton.clicked.connect(self._addApplication)
        self._removeButton.clicked.connect(self._removeApplication)
        self._upButton.clicked.connect(self._moveUp)
        self._downButton.clicked.connect(self._moveDown)
        self._externalAppList.itemDoubleClicked.connect(self._editApplication)
        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel("Press next button to continue")
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 1)

    def fromXml(self, xml):
        self._externalApps = []
        panelNodes = AppSettingsWizard.getArrayFromNode(
            xml, "ExternalApps", default=[])
        for child in panelNodes:
            name = AppSettingsWizard.getValueFromNode(
                child, "command", default=None)
            if name:
                self._externalApps.append((name, etree.tostring(child)))

    def _addApplication(self):
        name, xml, ok = ExternalAppEditor.getDialog()
        if ok:
            self._externalApps.append((name, xml))

        self._refreshApplicationList()

    def _editApplication(self):
        # edit
        self._refreshApplicationList()

    def _removeApplication(self):
        if len(self._externalAppList.selectedIndexes()) > 0:
            self._app_id = self._externalAppList.selectedIndexes()[0].row()
            self._externalApps.remove(self._externalApps[self._app_id])
            self._refreshApplicationList()

    def _moveUp(self):
        if len(self._externalAppList.selectedIndexes()) > 0:
            self._app_id = self._externalAppList.selectedIndexes()[0].row()
            if self._app_id > 0:
                tmp = self._externalApps[self._app_id]
                self._externalApps[self._app_id] = self._externalApps[
                    self._app_id - 1]
                self._externalApps[self._app_id - 1] = tmp
                self._refreshApplicationList()
                self._externalAppList.setCurrentIndex(self._externalAppList.indexFromItem(
                    self._externalAppList.item(self._app_id - 1)))

    def _moveDown(self):
        if len(self._externalAppList.selectedIndexes()) > 0:
            self._app_id = self._externalAppList.selectedIndexes()[0].row()
            if self._app_id < self._externalAppList.count() - 1:
                tmp = self._externalApps[self._app_id]
                self._externalApps[self._app_id] = self._externalApps[
                    self._app_id + 1]
                self._externalApps[self._app_id + 1] = tmp
                self._refreshApplicationList()
                self._externalAppList.setCurrentIndex(self._externalAppList.indexFromItem(
                    self._externalAppList.item(self._app_id + 1)))

    def _refreshApplicationList(self):
        self._externalAppList.clear()
        for panel in self._externalApps:
            name, xml = panel
            self._externalAppList.addItem(name)

    def _getExternalApps(self):
        if len(self._externalApps) <= 0:
            return None
        else:
            return self._externalApps

    def checkData(self):
        BasePage.checkData(self)
        self._valid = True

    def _setStatus(self, text):
        self._status_label.setText(text)


class MonitorPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)

    def initializePage(self):
        BasePage.initializePage(self)
        self.wizard().__setitem__("monitor", self._getMonitor)

    def _setupUI(self):
        BasePage._setupUI(self)
        self.setTitle('Monitor List')
        self._label = Qt.QLabel(
            "\nIf you want to monitor some attributes, add them to the monitor list. \n")
        self._label.setWordWrap(True)
        self._layout.addWidget(self._label, 0, 0, 1, 4)
        self._monitorLabel = Qt.QLabel("Monitor List:")
        self._monitorLineEdit = Qt.QLineEdit()
        self._monitorLineEdit.setToolTip(
            "Comma-separated list of attribute names")
        self._monitorLineEdit.setMinimumSize(400, 25)
        self._monitorLineEdit.setReadOnly(False)
        self._monitorButton = Qt.QPushButton()
        self._monitorButton.setToolTip("Browse...")
        # self._monitorButton.setIcon(Qt.QIcon.fromTheme("system-search"))
        self._monitorButton.setIcon(Qt.QIcon("designer:devs_tree.png"))
        self._monitorButton.setMaximumSize(80, 25)
        self._monitorClearButton = Qt.QPushButton()
        self._monitorClearButton.setToolTip("Clear")
        self._monitorClearButton.setMaximumSize(80, 25)
        self._monitorClearButton.setIcon(Qt.QIcon("actions:edit-clear.svg"))
        self._layout.addWidget(self._monitorLabel, 2, 0, Qt.Qt.AlignRight)
        self._layout.addWidget(self._monitorLineEdit, 2, 1, Qt.Qt.AlignRight)
        self._layout.addWidget(self._monitorButton, 2, 2, Qt.Qt.AlignLeft)
        self._layout.addWidget(self._monitorClearButton, 2, 3, Qt.Qt.AlignLeft)
        self._monitorButton.clicked.connect(self._selectMonitor)
        self._monitorClearButton.clicked.connect(self._clearMonitor)
        # self._synopticClear.hide()

        self._spacerItem1 = Qt.QSpacerItem(
            10, 0, Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Expanding)
        self._layout.addItem(self._spacerItem1, 8, 0, 1, 1, Qt.Qt.AlignCenter)
        self._status_label = Qt.QLabel("Press next button to continue")
        self.setStatusLabelPalette(self._status_label)
        self._layout.addWidget(self._status_label, 9, 0, 1, 4)

    def fromXml(self, xml):
        self._monitorLineEdit.setText(
            AppSettingsWizard.getValueFromNode(xml, "MONITOR", default=""))

    def _clearMonitor(self):
        self._monitorLineEdit.clear()
        # self._monitorClearButton.hide()

    def _getMonitor(self):
        return str(self._monitorLineEdit.text())

    def _selectMonitor(self):
        models, ok = taurus.qt.qtgui.panel.TaurusModelChooser.modelChooserDlg(
            host=None)
        if ok:
            self._monitorLineEdit.setText(",".join(models))
        self.checkData()

    def _setStatus(self, text):
        self._status_label.setText(text)


class OutroPage(BasePage):

    def __init__(self, parent=None):
        BasePage.__init__(self, parent)
        self._valid = True
        self.setTitle('Confirmation Page')
        self._label1 = Qt.QLabel("XML configuration file:")
        self._layout.addWidget(self._label1, 0, 0)
        self._xml = Qt.QTextEdit()
        self._xml.setSizePolicy(Qt.QSizePolicy(
            Qt.QSizePolicy.Expanding, Qt.QSizePolicy.Expanding))
        self._layout.addWidget(self._xml, 1, 0)
        self._label2 = Qt.QLabel("Files copied")
        self._layout.addWidget(self._label2, 2, 0)
        self._substTable = Qt.QTableWidget()
        self._substTable.setColumnCount(2)
        self._substTable.setEditTriggers(self._substTable.NoEditTriggers)
        self._substTable.setHorizontalHeaderLabels(
            ('Original file', 'File in Project dir'))
        self._substTable.setSizePolicy(Qt.QSizePolicy(
            Qt.QSizePolicy.Expanding, Qt.QSizePolicy.Expanding))
        self._layout.addWidget(self._substTable, 3, 0)

    def _getXml(self):
        return str(self._xml.toPlainText())

    def saveFile(self, fileName):
        file = Qt.QFile(fileName)

        if not file.open(Qt.QFile.WriteOnly | Qt.QFile.Text):
            Qt.QMessageBox.warning(self, self.tr("Saving XML..."),
                                   self.tr("Cannot write file %1:\n%2.")
                                   .arg(fileName)
                                   .arg(file.errorString()))
            return False

        file.write(str(self._xml.toPlainText()))
        self._valid = True
        self.checkData()
        file.close()

        return True

    def initializePage(self):
        xml, substitutions = self.wizard().generateXml()
        self._xml.setText(xml)
        self.wizard().__setitem__("xml", self._getXml)
        self._substTable.clearContents()
        self._substTable.setRowCount(len(substitutions))
        for i, (dst, src) in enumerate(substitutions.items()):
            item0, item1 = Qt.QTableWidgetItem(src), Qt.QTableWidgetItem(dst)
            self._substTable.setItem(i, 0, item0)
            self._substTable.setItem(i, 1, item1)
        self._substTable.resizeColumnsToContents()

    def validatePage(self):
        try:
            self.createProject()
        except Exception as e:
            Qt.QMessageBox.warning(self, 'Error creating project',
                                   'Could not create project files. \nReason:%s' % repr(
                                       e),
                                   Qt.QMessageBox.Cancel)
            import traceback
            traceback.print_exc()
            return False
        return True

    def createProject(self):
        # prepare a log file
        pdir = self.wizard().__getitem__('projectDir')
        gui_name = self.wizard().__getitem__("guiName")
        install_dir = os.path.join(pdir, "tgconf_{0}".format(gui_name))
        if not os.path.exists(install_dir):
            os.makedirs(install_dir)
        logfilename = os.path.join(pdir, 'wizard.log')
        logfile = open(logfilename, 'w')
        logfile.write('Project created by AppSettingsWizard on %s\n' %
                      datetime.datetime.now().isoformat())
        # copy files
        for i in range(self._substTable.rowCount()):
            src = str(self._substTable.item(i, 0).text())
            dst = os.path.join(install_dir, str(
                self._substTable.item(i, 1).text()))
            if os.path.normpath(src) != os.path.normpath(dst):
                shutil.copy(src, dst)
                logfile.write('File copied: %s --> %s\n' % (src, dst))
        # write xml config file
        xmlcfgfilename = os.path.join(install_dir,
                                      self.wizard().getXmlConfigFileName())
        f = open(xmlcfgfilename, 'w')
        f.write(str(self._xml.toPlainText()))
        f.close()
        logfile.write('XML Config file created: "%s"\n' % xmlcfgfilename)
        # write python config file
        pycfgfilename = os.path.join(install_dir,
                                '%s.py' % self.wizard().getConfigFilePrefix())
        f = open(pycfgfilename, 'w')
        f.write("XML_CONFIG = '%s'" % self.wizard().getXmlConfigFileName())
        f.close()
        logfile.write('Python config file created: "%s"\n' % pycfgfilename)
        # write __init__.py config file
        init_template = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'res', 'init.template')
        f = open(init_template, 'r')
        template = f.read()
        f.close()
        initfilename = os.path.join(install_dir, '__init__.py')
        f = open(initfilename, 'w')
        template = template.format(name=gui_name)
        f.write(template)
        f.close()
        logfile.write('python init file created: "%s"\n' % initfilename)
        # write setup file
        setup_template = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'res', 'setup.template')
        f = open(setup_template, 'r')
        template = f.read()
        f.close()
        setup = os.path.join(pdir, "setup.py")
        f = open(setup, 'w')
        template = template.format(name=gui_name)
        f.write(template)
        f.close()
        # write MANIFEST.in file
        manifest_template = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'res', 'manifest.template')
        f = open(manifest_template, 'r')
        template = f.read()
        f.close()
        manifestfile = os.path.join(pdir, "MANIFEST.in")
        f = open(manifestfile, 'w')
        template = template.format(name=gui_name)
        f.write(template)
        f.close()
        # if all went ok...
        msg = 'Application project was successfully created.' +\
              'You can find the files in: "%s"' % pdir
        msg += '\nTip: You can install it with:\n\tpip install %s' % pdir
        msg += '\nTip: And then run the application with:\n\t %s' % gui_name
        details = ''
        warnings = self.wizard().getProjectWarnings()
        if warnings:
            msg += '\n\nHowever, some fine-tuning may be needed. Please check the details:\n'
            for _short, _long in warnings:
                details += '- %s: %s\n\n' % (_short, _long)
        logfile.write(msg + details)
        logfile.close()
        dlg = Qt.QMessageBox(Qt.QMessageBox.Information,
                             'Application project created', msg, Qt.QMessageBox.Ok, self)
        dlg.setDetailedText(details)
        dlg.exec_()
        print() 
        print(msg + details)
        print()


class AppSettingsWizard(Qt.QWizard):
    """
    This Wizard provide functionality for creating from scratch a configuration
    directory for a TaurusGUI based application.

    The files in the configuration dir determine the default, permanent, pre-defined
    contents of the GUI. While the user may add/remove more elements at run
    time and those customizations will also be stored, this file defines what a
    user will find when launching the GUI for the first time.
    """
    Pages = Enumeration('Pages', ('IntroPage', 'ProjectPage', 'GeneralSettings', 'CustomLogoPage', 'SynopticPage',
                                  'MacroServerInfo', 'InstrumentsPage', 'PanelsPage', 'ExternalAppPage', 'MonitorPage', 'OutroPage'))
    SARDANA_INSTALLED = False

    def __init__(self, parent=None, jdrawCommand='jdraw', configFilePrefix='config'):
        Qt.QWizard.__init__(self, parent)
        self.installEventFilter(self)
        self._item_funcs = {}
        self._pages = {}
        self._jdrawCommand = jdrawCommand
        self._configFilePrefix = configFilePrefix
        self._loadPages()
        self._substitutions = {}
        self._projectWarnings = []

    def getProjectWarnings(self):
        return self._projectWarnings

    def getConfigFilePrefix(self):
        return self._configFilePrefix

    def getXmlConfigFileName(self):
        return "%s.xml" % self._configFilePrefix

    @staticmethod
    def getValueFromNode(rootNode, nodeName, default=None):
        '''
        returns a value from given Node
        :param rootNode: (etree.Element) root node
        :param nodeName: the name of node to find
        :param default: returned value if node is None or contains empty string
        '''
        node = rootNode.find(nodeName)
        if (node is not None) and (node.text is not None):
            return node.text
        else:
            return default

    @staticmethod
    def getArrayFromNode(rootNode, nodeName, default=None):
        '''
        returns an array contained by given Node
        :param rootNode: (etree.Element) root node
        :param nodeName: the name of node to find
        :param default: returned value if node is None or contains empty string
        '''
        array = []
        node = rootNode.find(nodeName)
        if (node is not None) and (node.text is not None):
            for child in node:
                array.append(child)
            return array
        else:
            return default

    def loadXml(self, fname):
        '''
        parses xml code and sets all pages according to its contents. It
        raises an exception if something could not be processed

        :param fname: (unicode) path to file containing xml code
        '''
        projectDir, cfgfile = os.path.split(fname)
        f = open(fname, 'r')
        xml = f.read()
        root = etree.fromstring(xml)

        # print self.Pages
        for pageNumber in range(len(self.Pages)):
            self.page(pageNumber).fromXml(root)

    def getXml(self):
        try:
            return self.__getitem__("xml")
        except Exception as e:
            return None

    def __setitem__(self, name, value):
        self._item_funcs[name] = value

    def __getitem__(self, name):
        for id in self.getPages():
            p = self.page(id)
            if isinstance(p, BasePage):
                try:
                    return p[name]()
                except Exception as e:
                    pass
        return self._item_funcs[name]()

    def addPage(self, page):
        id = Qt.QWizard.addPage(self, page)
        self._pages[id] = page

    def setPage(self, id, page):
        Qt.QWizard.setPage(self, id, page)
        self._pages[id] = page

    def getPages(self):
        return self._pages

    def _loadPages(self):
        intro = IntroPage()
        self.setPage(self.Pages.IntroPage, intro)
        intro.setNextPageId(self.Pages.ProjectPage)

        project_page = ProjectPage()
        self.setPage(self.Pages.ProjectPage, project_page)
        project_page.setNextPageId(self.Pages.GeneralSettings)

        general_settings_page = GeneralSettings()
        self.setPage(self.Pages.GeneralSettings, general_settings_page)
        general_settings_page.setNextPageId(self.Pages.CustomLogoPage)

        custom_logo_page = CustomLogoPage()
        self.setPage(self.Pages.CustomLogoPage, custom_logo_page)
        custom_logo_page.setNextPageId(self.Pages.SynopticPage)

        synoptic_page = SynopticPage()
        self.setPage(self.Pages.SynopticPage, synoptic_page)

        try:
            from sardana.taurus.qt.qtgui.extra_macroexecutor.common import \
                TaurusMacroConfigurationDialog
            # try to instantiate the dialog (e.g. this fails if using Qt5 with
            # versions of sardana which do not support it)
            _ = TaurusMacroConfigurationDialog()
            self.SARDANA_INSTALLED = True
        except:
            self.SARDANA_INSTALLED = False
        if self.SARDANA_INSTALLED:
            synoptic_page.setNextPageId(self.Pages.MacroServerInfo)

            macroserver_page = MacroServerInfoPage()
            self.setPage(self.Pages.MacroServerInfo, macroserver_page)
            macroserver_page.setNextPageId(self.Pages.InstrumentsPage)

            instruments_page = InstrumentsPage()
            self.setPage(self.Pages.InstrumentsPage, instruments_page)
            instruments_page.setNextPageId(self.Pages.PanelsPage)

        else:
            synoptic_page.setNextPageId(self.Pages.PanelsPage)

        panels_page = PanelsPage()
        self.setPage(self.Pages.PanelsPage, panels_page)
        panels_page.setNextPageId(self.Pages.ExternalAppPage)

        external_app_page = ExternalAppPage()
        self.setPage(self.Pages.ExternalAppPage, external_app_page)
        external_app_page.setNextPageId(self.Pages.MonitorPage)

        monitor_page = MonitorPage()
        self.setPage(self.Pages.MonitorPage, monitor_page)
        monitor_page.setNextPageId(self.Pages.OutroPage)

        outro_page = OutroPage()
        self.setPage(self.Pages.OutroPage, outro_page)
        outro_page.setNextPageId(-1)

        self.setOption(Qt.QWizard.CancelButtonOnLeft, True)

    def generateXml(self):
        '''returns the xml code corresponding to the options selected in the wizard
        and a dictionary representing the paths that have been substituted.

        :return: (str, dict<str,str>) The return value is a tuple whose first element
                 is the xml code and the second element is a dict where the keys are the
                 destination files and the values are the original paths.
        '''
        root = etree.Element("taurusgui_config")
        # general settings page
        guiName = etree.SubElement(root, "GUI_NAME")
        guiName.text = self.__getitem__("guiName")
        organizationName = etree.SubElement(root, "ORGANIZATION")
        organizationName.text = self.__getitem__("organizationName")
        # custom logo page
        customLogo = etree.SubElement(root, "CUSTOM_LOGO")
        src = self.__getitem__("customLogo")
        mod_dir = os.path.join(self.__getitem__('projectDir'),
                            'tgconf_' + guiName.text)
        mod_dir = os.path.abspath(mod_dir)  # make sure mod_dir is absolute
        if src is None or ":" in src:
            # using registered paths
            # TODO: what if they use windows paths such as "C:\foo" ?
            dst = src
        else:
            # if src is absolute, it stays so, and if it is relative, we assume
            # mod_dir as the root dir
            src = os.path.join(mod_dir, src)
            dst = os.path.basename(self.substitutionName(src, mod_dir))
        customLogo.text = dst
        # synoptic page
        synopticList = self.__getitem__("synoptics")
        if synopticList:
            synoptics = etree.SubElement(root, "SYNOPTIC")
            for src in synopticList:
                src = os.path.join(mod_dir, src)
                dst = self.substitutionName(src, mod_dir)
                child = etree.SubElement(synoptics, "synoptic",
                                         str=os.path.basename(dst))
                # substitute any referenced files within the jdrawfiles
                f = open(src, 'r')
                contents = f.read()
                f.close()
                for ref in re.findall(r'file_name:\"(.+?)\"', contents):
                    # this is ok for both relative and absolute references
                    refsrc = os.path.join(os.path.dirname(src), ref)
                    refdst = self.substitutionName(refsrc, mod_dir)
                    if ref != refdst:
                        _short = 'Manual editing needed in "%s"' % dst
                        _long = ('The synoptic file "%s" references a file that '
                                'has been copied to the project dir in order to make the project portable. '
                                'Please edit "%s" and replace "%s" by "%s"') % (dst, dst, ref, refdst)
                        self._projectWarnings.append((_short, _long))

        # macroserver page
        if self.SARDANA_INSTALLED and self.__getitem__("macroServerName"):
            macroServerName = etree.SubElement(root, "MACROSERVER_NAME")
            macroServerName.text = self.__getitem__("macroServerName")
            doorName = etree.SubElement(root, "DOOR_NAME")
            doorName.text = self.__getitem__("doorName")
            instruments = etree.SubElement(root, "INSTRUMENTS_FROM_POOL")
            instruments.text = str(self.__getitem__("instruments"))
        # panels page
        panelList = self.__getitem__("panels")
        if panelList:
            panels = etree.SubElement(root, "PanelDescriptions")
            for panel in panelList:
                name, xml = panel
                item = etree.fromstring(xml)
                panels.append(item)
        # external apps page
        externalAppList = self.__getitem__("externalApps")
        if externalAppList:
            externalApps = etree.SubElement(root, "ExternalApps")
            for externalApp in externalAppList:
                name, xml = externalApp
                item = etree.fromstring(xml)
                externalApps.append(item)
        # monitor page
        monitor = etree.SubElement(root, "MONITOR")
        monitor.text = self.__getitem__("monitor")

        return etree.tostring(root, pretty_print=True), copy.copy(self._substitutions)

    def substitutionName(self, src, mod_dir):
        name = os.path.basename(src)
        i = 2
        if os.path.dirname(os.path.abspath(src)) != os.path.abspath(mod_dir):
            # do not change the name if it is the same dir!
            while name in self._substitutions:
                root, ext = os.path.splitext(name)
                name = "%s_%i%s" % (root, i, ext)
                i += 1
        self._substitutions[name] = src
        return name


def main():
    app = Qt.QApplication([])
    wizard = AppSettingsWizard()
    wizard.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()



# !/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a set of dialog based widgets"""

__all__ = ["TaurusConfigurationDialog"]

__docformat__ = 'restructuredtext'

from taurus.external.qt import Qt
from taurus.qt.qtgui.panel.taurusconfigurationpanel import TaurusConfigurationPanel


class TaurusConfigurationDialog(Qt.QDialog):

    def __init__(self, parent=None, designMode=False):
        Qt.QDialog.__init__(self, parent)
        self.setWindowTitle('TaurusConfigurationDialog')
        layout = Qt.QVBoxLayout()
        self.setLayout(layout)
        ConfigPanel = TaurusConfigurationPanel
        self._panel = ConfigPanel(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self._panel)
        self._panel._ui.pushButtonOk.setVisible(True)
        self._panel._ui.pushButtonCancel.setVisible(True)
        self._panel._ui.pushButtonOk.clicked.connect(self._onOk)
        self._panel._ui.pushButtonCancel.clicked.connect(self._onCancel)
        self.adjustSize()
        self.show()

    def _onOk(self):
        self._panel._onOk()
        self._onCancel()

    def _onCancel(self):
        self.close()

    def setModel(self, model):
        self._panel.setModel(model)


def main():
    import sys
    attr_name = sys.argv[1]
    a = Qt.QApplication([])
    d = TaurusConfigurationDialog()
    d.setModel(attr_name)
    return a.exec_()

if __name__ == "__main__":
    import sys
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
###########################################################################

''' Creates a tree of dirs and restructured text stub files for documenting
the API of a python module with sphinx'''
from __future__ import print_function
from builtins import zip
from builtins import object
import sys
import os
import imp
from jinja2 import Environment, FileSystemLoader


def taurusabspath(*path):
    """A method to determine absolute path for a given relative path to the
    directory where the setup.py script is located"""
    this_dir = os.path.dirname(os.path.abspath(__file__))
    setup_dir = os.path.abspath(os.path.join(this_dir, os.pardir))
    return os.path.join(setup_dir, *path)

# import moduleexplorer from the sources, and without importing taurus
__name = "moduleexplorer"
__path = taurusabspath('lib', 'taurus', 'test', 'moduleexplorer.py')
ModuleExplorer = imp.load_source(__name, __path).ModuleExplorer


class Auto_rst4API_Creator(object):
    AUTOGEN_SIGNATURE = '.. AUTO_RST4API'
    AUTOGEN_MESSAGE = '.. This file was generated by auto_rst4api.py. Changes may be lost'

    def __init__(self, templatespath='./', moduletemplate='api_module.rst', classtemplate='api_class.rst',
                 classindextemplate='api_AllClasses.rst', exclude_patterns=(), verbose=True, overwrite_old=False):
        '''
        :param templates: (str) path to dir where template files are located
        :param moduletemplate: (str) name of the template to be used for module pages
        :param classtemplate: (str) name of the template to be used for class pages
        :param classindextemplate: (str) name of the template to be used for class index page
        :param verbose: (bool) If True (default) status messages will be printed to stdout
        '''

        self.verbose = verbose
        self.exclude_patterns = exclude_patterns

        self.env = Environment(loader=FileSystemLoader(templatespath))
        self.moduletemplate = self.env.get_template(moduletemplate)
        self.classtemplate = self.env.get_template(classtemplate)
        self.classindextemplate = self.env.get_template(classindextemplate)
        self.overwrite_old = overwrite_old

    def _isautogeneratedfile(self, fname):
        ret = False
        f = open(fname, 'r')
        lines = f.readlines()
        for l in lines:
            if l.startswith(self.AUTOGEN_SIGNATURE):
                ret = True
                break
        f.close()

        return ret

    def cleanAutogenerated(self, apipath):
        '''Removes any previously autogenerated rst file in the given path
        or its subdirectories

        :param apipath: (str) directory to clean
        '''
        for dirpath, dirnames, filenames in os.walk(apipath):
            for f in filenames:
                if f.endswith('.rst'):
                    fullname = os.path.join(dirpath, f)
                    try:
                        if self._isautogeneratedfile(fullname):
                            print("Removing %s" % fullname)
                            os.remove(fullname)
                    except Exception as e:
                        print('Error accessing %s:%s' % (fullname, repr(e)))

    def createClassIndex(self, info, ofname):
        '''
        Creates a class index page using the classindextemplate.

        :param info: (dict) dictionary containing the information about the
                     items to document for this module (as generated by
                     :meth:`exploreModule`
        :param ofname: (str) output file name
        '''
        classes = ModuleExplorer.getAll(
            info, 'localclassnames')  # this is a list of tuples of (modulename,class)
        classes = sorted(classes, key=lambda item: item[
                         1])  # sort it by class name
        classes = ['.'.join((m, c))
                   for m, c in classes]  # make a full classname
        if self.verbose:
            print('creating "%s" ...' % ofname, end=' ')
        if not os.path.exists(ofname) or (self.overwrite_old and self._isautogeneratedfile(ofname)):
            text = self.classindextemplate.render(info=info, classes=classes)
            f = open(ofname, "w")
            f.write('\n'.join((self.AUTOGEN_SIGNATURE, self.AUTOGEN_MESSAGE, text)))
            f.close()
            if self.verbose:
                print(' ok.')
        else:
            if self.verbose:
                print(' skipping (file already exists)')

    def createStubs(self, info, docparentpath):
        '''creates rst stub files for modules and classes according to the
        information contained in info.

        :param info: (dict) dictionary containing the information about the
                     items to document for this module (as generated by
                     :meth:`exploreModule`)
        :docparentpath: (str) path to the directory in which the documentation
                        files will be written
        '''
        # create the module doc dir if it didn't exist
        absdocpath = os.path.join(docparentpath, info['basemodulename'])
        if not os.path.exists(absdocpath):
            os.makedirs(absdocpath, mode=0o755)
        # create module index stub in doc parent dir
        ofname = os.path.join(docparentpath, "%s.rst" % info['basemodulename'])
        if self.verbose:
            print('creating "%s" ...' % ofname, end=' ')
        if not os.path.exists(ofname) or (self.overwrite_old and self._isautogeneratedfile(ofname)):
            text = self.moduletemplate.render(info=info)
            f = open(ofname, "w")
            f.write('\n'.join((self.AUTOGEN_SIGNATURE, self.AUTOGEN_MESSAGE, text)))
            f.close()
            if self.verbose:
                print(' ok.')
        else:
            if self.verbose:
                print(' skipping (file already exists)')
        # create class stubs
        for name in info['localclassnames']:
            ofname = os.path.join(absdocpath, "_%s.rst" % name)
            if self.verbose:
                print('creating "%s" ...' % ofname, end=' ')
            if not os.path.exists(ofname) or (self.overwrite_old and self._isautogeneratedfile(ofname)):
                text = self.classtemplate.render(info=info, classname=name)
                f = open(ofname, "w")
                f.write(
                    '\n'.join((self.AUTOGEN_SIGNATURE, self.AUTOGEN_MESSAGE, text)))
                f.close()
                if self.verbose:
                    print(' ok.')
            else:
                if self.verbose:
                    print(' skipping (file already exists)')
        # recurse for submodules
        for sminfo in info['submodules'].values():
            self.createStubs(sminfo, absdocpath)

    def documentModule(self, modulename, docparentpath, exclude_patterns=None):
        '''
        recursive function that walks on the module structure and generates
        documentation files for the given module and its submodules. It also
        creates a class index for the root module

        :param modulename: (str) name of the module to document
        :docparentpath: (str) path to the directory in which the documentation
                        files will be written
        :param exclude_patterns: (seq<str>) sequence of strings containing regexp
                 patterns. Each candidate to be documented will be
                 matched against these patterns and will be excluded
                 if it matches any of them.

        :return: (list<str>) list of warning messages
        '''
        if self.verbose:
            print("\nDocumenting %s..." % modulename)
        if exclude_patterns is None:
            exclude_patterns = self.exclude_patterns
        moduleinfo, w = ModuleExplorer.explore(modulename,
                                               exclude_patterns=exclude_patterns,
                                               verbose=self.verbose)
        self.createStubs(moduleinfo, docparentpath)
        self.createClassIndex(moduleinfo, os.path.join(
            docparentpath, "%s_AllClasses.rst" % modulename))
        if len(w) == 0:
            return []
        else:
            return list(zip(*w))[1]


def main():
    import sys
    if len(sys.argv) != 3:
        print('Usage:\n\t%s modulename docpreffix\n\n' % sys.argv[0])
        sys.exit(1)
    modulename, docparentpath = sys.argv[1:]
    creator = Auto_rst4API_Creator(verbose=True)
    r = creator.documentModule(
        modulename, docparentpath, exclude_patterns=['.*\.test'])
    print('\n\n' + '*' * 50)
    print("Auto Creation of API docs for %s Finished with %i warnings:" % (modulename, len(r)))
    print('\n'.join(r))
    print('*' * 50 + '\n')

if __name__ == "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Extension of :mod:`guiqwt.styles`"""


__docformat__ = 'restructuredtext'


from guidata.dataset.datatypes import DataSet
from guidata.dataset.dataitems import StringItem, IntItem, ChoiceItem, BoolItem


# ===================================================
# Taurus Curve parameters
# ===================================================
class TaurusCurveParam(DataSet):
    xModel = StringItem("Model for X", default="")
    yModel = StringItem("Model for Y", default="")

    def update_param(self, curve):
        self.xModel.update_param(curve.taurusparam.xModel)
        self.yModel.update_param(curve.taurusparam.yModel)
#        DataSet.update_param(self, curve)

    def update_curve(self, curve):
        curve.setModels(self.xModel or None, self.yModel)
#        DataSet.update_curve(self, curve)


class TaurusTrendParam(DataSet):
    model = StringItem("Model", default="")
    maxBufferSize = IntItem("Buffer Size", default=16384)
    useArchiving = BoolItem("Use Archiving", default=False)
    stackMode = ChoiceItem("Stack Mode",
                           [("datetime",  "Absolute Time"),
                            ("timedelta", "Relative Time"),
                               ("event", "Event")],
                           default="datetime")

    def update_param(self, curve):
        self.model.update_param(curve.taurusparam.model)
        self.maxBufferSize.update_param(curve.taurusparam.maxBufferSize)
        self.stackMode.update_param(curve.taurusparam.stackMode)

    def update_curve(self, curve):
        curve.setModel(self.model)
        curve.setBufferSize(self.maxBufferSize)
        # the stackMode is directly used from the param, so there is no need to
        # update



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
tauruspluginplugin.py:
"""
from __future__ import absolute_import

from taurus.external.qt import QtDesigner


def build_qtdesigner_widget_plugin(klass):
    #from . import taurusplugin # - after futurize stage 1
    import taurusplugin.taurusplugin as taurusplugin # + after futurize stage 1

    class Plugin(taurusplugin.TaurusWidgetPlugin):
        WidgetClass = klass

    Plugin.__name__ = klass.__name__ + "QtDesignerPlugin"
    return Plugin

_SKIP = ["QLogo", "QGroupWidget", "TaurusGroupWidget"]

_plugins = {}


def main():
    from taurus.core.util.log import Logger
    from taurus.qt.qtgui.util import TaurusWidgetFactory
    Logger.setLogLevel(Logger.Debug)
    _log = Logger(__name__)

    try:
        wf = TaurusWidgetFactory()
        klasses = wf.getWidgetClasses()
        ok_nb, skipped_nb, e1_nb, e2_nb, e3_nb, e4_nb = 0, 0, 0, 0, 0, 0
        for widget_klass in klasses:
            name = widget_klass.__name__
            #_log.debug("Processing %s" % name)
            if name in _SKIP:
                #_log.debug("Skipped %s" % name)
                skipped_nb += 1
                continue
            # if getQtDesignerPluginInfo does not exist, returns None or raises
            # an exception: forget the widget
            cont = False
            try:
                qt_info = widget_klass.getQtDesignerPluginInfo()
                if qt_info is None:
                    #_log.debug("E1: Canceled %s (getQtDesignerPluginInfo)" % name)
                    e1_nb += 1
                    cont = True
            except AttributeError:
                #_log.debug("E2: Canceled %s (widget doesn't have getQtDesignerPluginInfo())" % name)
                e2_nb += 1
                cont = True
            except Exception as e:
                #_log.debug("E3: Canceled %s (%s)" % (name, str(e)))
                e3_nb += 1
                cont = True

            if cont:
                continue
            for k in ('module', ):
                if k not in qt_info:
                    #_log.debug("E4: Canceled %s (getQtDesignerPluginInfo doesn't have key %s)" % (name, k))
                    e4_nb += 1
                    cont = True
            if cont:
                continue

            plugin_klass = build_qtdesigner_widget_plugin(widget_klass)
            plugin_klass_name = plugin_klass.__name__
            globals()[plugin_klass_name] = plugin_klass
            _plugins[plugin_klass_name] = plugin_klass

            ok_nb += 1
            #_log.debug("DONE processing %s" % name)
        _log.info("Inpected %d widgets. %d (OK), %d (Skipped), %d (E1), %d (E2), %d (E3), %d(E4)" % (
            len(klasses), ok_nb, skipped_nb, e1_nb, e2_nb, e3_nb, e4_nb))
        _log.info("E1: getQtDesignerPluginInfo() returns None")
        _log.info("E2: widget doesn't implement getQtDesignerPluginInfo()")
        _log.info("E3: getQtDesignerPluginInfo() throws exception")
        _log.info(
            "E4: getQtDesignerPluginInfo() returns dictionary with missing key (probably 'module' key)")
    except Exception as e:
        import traceback
        traceback.print_exc()
        # print e


class TaurusWidgets(QtDesigner.QPyDesignerCustomWidgetCollectionPlugin):

    def __init__(self, parent=None):
        QtDesigner.QPyDesignerCustomWidgetCollectionPlugin.__init__(parent)
        self._widgets = None

    def customWidgets(self):
        if self._widgets is None:
            self._widgets = [w(self) for w in _plugins.values()]
        return self._widgets

if __name__ != "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a set of basic taurus widgets based on QWheelEdit"""

from __future__ import absolute_import

from taurus.external.qt import Qt

from taurus.core.taurusbasetypes import TaurusEventType
from taurus.qt.qtgui.base import TaurusBaseWritableWidget
from .qwheel import QWheelEdit


__all__ = ["TaurusWheelEdit"]

__docformat__ = 'restructuredtext'


class TaurusWheelEdit(QWheelEdit, TaurusBaseWritableWidget):

    def __init__(self, qt_parent=None, designMode=False):
        name = self.__class__.__name__
        self.call__init__wo_kw(QWheelEdit, qt_parent)
        self.call__init__(TaurusBaseWritableWidget,
                          name, designMode=designMode)
        self.numberChanged.connect(self.notifyValueChanged)
        self.returnPressed.connect(self.writeValue)
        self.valueChangedSignal.connect(self.updatePendingOperations)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TaurusBaseWidget overwriting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def handleEvent(self, evt_src, evt_type, evt_value):
        if evt_type == TaurusEventType.Config and evt_value is not None:
            import re
            # match the format string to "%[width][.precision][f_type]"
            obj = self.getModelObj()
            m = re.match(r'%([0-9]+)?(\.([0-9]+))?([df])', obj.format)
            if m is None:
                raise ValueError("'%s' format unsupported" % obj.format)

            width, _, precision, f_type = m.groups()

            if width is None:
                width = self.DefaultIntDigitCount + \
                    self.DefaultDecDigitCount + 1
            else:
                width = int(width)

            if precision is None:
                precision = self.DefaultDecDigitCount
            else:
                precision = int(precision)

            dec_nb = precision

            if dec_nb == 0 or f_type == 'd':
                int_nb = width
            else:
                int_nb = width - dec_nb - 1  # account for decimal sep

            self.setDigitCount(int_nb=int_nb, dec_nb=dec_nb)
            try:
                self.setMinValue(float(obj.min_value))
            except:
                pass
            try:
                self.setMaxValue(float(obj.max_value))
            except:
                pass
        TaurusBaseWritableWidget.handleEvent(
            self, evt_src, evt_type, evt_value)

    def updateStyle(self):
        TaurusBaseWritableWidget.updateStyle(self)
        if self.hasPendingOperations():
            self.setStyleSheet(
                'QWheelEdit {border: 2px solid; border-radius: 4px; border-color: blue}')
        else:
            self.setStyleSheet(
                'QWheelEdit {border: 2px solid; border-radius: 4px; border-color: rgba(0,0,255,0)}')

    @classmethod
    def getQtDesignerPluginInfo(cls):
        ret = TaurusBaseWritableWidget.getQtDesignerPluginInfo()
        ret['module'] = 'taurus.qt.qtgui.input'
        ret['icon'] = "designer:wheeledit.png"
        return ret

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    model = Qt.pyqtProperty("QString", TaurusBaseWritableWidget.getModel,
                            TaurusBaseWritableWidget.setModel,
                            TaurusBaseWritableWidget.resetModel)

    useParentModel = Qt.pyqtProperty("bool", TaurusBaseWritableWidget.getUseParentModel,
                                     TaurusBaseWritableWidget.setUseParentModel,
                                     TaurusBaseWritableWidget.resetUseParentModel)

    autoApply = Qt.pyqtProperty("bool", TaurusBaseWritableWidget.getAutoApply,
                                TaurusBaseWritableWidget.setAutoApply,
                                TaurusBaseWritableWidget.resetAutoApply)

    forcedApply = Qt.pyqtProperty("bool", TaurusBaseWritableWidget.getForcedApply,
                                  TaurusBaseWritableWidget.setForcedApply,
                                  TaurusBaseWritableWidget.resetForcedApply)



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Test util module for creating test cases for name validators"""

#__all__ = []

from builtins import object
from functools import partial
from taurus.test import insertTest
from taurus.core.taurusvalidator import TaurusAttributeNameValidator

__docformat__ = 'restructuredtext'


valid = partial(insertTest, helper_name='isValid')
invalid = partial(insertTest, helper_name='isInvalid')
names = partial(insertTest, helper_name='getNames')


class AbstractNameValidatorTestCase(object):
    '''A util class to create test cases for name validators'''
    validator = None  # implement in derived classes

    def isValid(self, name=None, groups=None, strict=True):
        msg = '%s should be valid (with strict=%s)' % (name, strict)
        self.assertTrue(self.validator().isValid(name, strict=strict), msg)
        if groups is not None:
            returned = self.validator().getUriGroups(name, strict=strict)
            for k, v in groups.items():
                msg = ('"%s" not in %s.getUriGroups("%s"). Returned %s' %
                       (k, self.validator.__name__, name, returned))
                self.assertIn(k, returned, msg=msg)
                msg = ('%s.getUriGroups("%s")["%s"] should be "%s". ' +
                       'Returned "%s"\nGroups: %s') % (self.validator.__name__,
                                                       name, k, v, returned[k],
                                                       str(returned))
                self.assertEqual(v, returned[k], msg=msg)

    def isInvalid(self, name=None, groups=None, strict=True,
                  exceptionType=None):
        try:
            valid = self.validator().isValid(name, strict=strict)
            groups = self.validator().getUriGroups(name, strict=strict)
            msg = '%s should be invalid. Matched: %s' % (name, groups)
            self.assertFalse(valid, msg)
        except exceptionType:
            # do not fail if the exception is expected
            pass
        else:
            # there was no exception. Check if an exception was expected
            msg = '%s should raise %s' % (name, exceptionType)
            self.assertTrue(exceptionType is None, msg)

    def test_singleton(self):
        '''Check that the validator is a singleton'''
        self.assertIs(self.validator(), self.validator())

    def getNames(self, name=None, out=None):
        v = self.validator()
        if isinstance(v, TaurusAttributeNameValidator):
            fragment = len(out) > 3
            names = v.getNames(name, fragment=fragment)
        else:
            names = v.getNames(name)
        self.assertEqual(names, out)
